﻿/*
Название: ImposeMultiPage.jsx
Приложение для использования: Adobe InDesign CS3, CS4, CS5, CS6
Версия: 1.3
Язык реализации (Среда): JavaScript (ExtendScript Toolkit 2)
Операционные системы (Платформы): PC, Macintosh (Windows, Mac OS)
Условия распространения: Бесплатно; На Ваш риск
Назначение: Помещает страницы многостраничного графического файла в текстовые фреймы,
цифры в которых соответствуют номерам страниц многостраничного документа.
Текстовые фреймы, предназначенные для операции, должные быть отмечены 
соответствующим объектным стилем, или цифрам во фреймах должен быть
присвоен соответствующий стиль абзаца или стиль символа. Также имеется 
опция "Любой тип маркирующего стиля" и "Обработка только выделенного".
Виды многостраничных документов, пригодные для операции: 
PDF (.pdf), документ Adobe Illustrator (.ai, имеющий PDF-совместимость), документ Adobe InDesign (.indd)

ФУНКЦИОНАЛЬНЫЕ ОГРАНИЧЕНИЯ: 
-------------------------------------------------------
Не контролируется диапазон цифр в текстовых фреймах и реальное количество страниц  
в помещаемом графическом файле (например, если в текстовом фрейме указана цифра 10, 
а количество страниц в помещаемом графическом файле 5, тогда во фрейм будет
помещена некорректная страница графического файла).
Обрабатываемые мастер-элементы (текстовые фреймы) на обычных страницах документа не могут входить в
сложные объекты (не должны состоять в группах и т.п.), они должны быть отдельными объктами.
Не обрабатываются мастер-элементы на мастер-разворотах.
Текстовые фреймы, содержащие цифры для номеров помещаемых графических страниц,
не должны входить в текстовые цепочки (эти фреймы должны быть отдельными текстовыми фреймами).
Не обрабатываются текстовые фреймы, границы которых выходят за границы пейстборда.
Не обрабатываются текстовые фреймы, находящиеся в переполненном (невидимом) тексте.
Если в текстовых фреймах используются символы номера текущей страницы, стиль нумерации должен
быть основан на обычных арабских цифрах.

Техническая поддержка: Sergey-Anosov@yandex.ru
https://sites.google.com/site/dtpscripting

================================================================================

Name: ImposeMultiPage.jsx
Application to use with: Adobe InDesign CS3, CS4, CS5, CS6
Version: 1.3
Program language (Environment): JavaScript (ExtendScript Toolkit 2)
Operating systems (Platforms): PC, Macintosh (Windows, Mac OS)
Distribution conditions: Freeware; At your own risk
Functions: Places graphic pages of multi-page graphic file into text frames, which are containing
numbers. The text frames must be marked with a marking object style, or numbers inside ones 
must have appropriate paragraph style or character style. Also options "Any style type" and 
"Process selected only" are available. Types of milti-page documents: PDF (.pdf), 
Adobe Illustrator document (.ai, PDF-compatible), Adobe InDesign document (.indd).

FUNCTIONAL LIMITATIONS:
---------------------------------------
Does not control the range of numbers in text frames and the real quantity of graphic pages in the file is being placed
(for instance, if a text frame contents number 10, and the quantity of graphic pages in the file is 5, 
then incorrect graphic page will be placed into the frame).
Processed master items (text frames) on ordinar pages of document must not belong 
to complex objects (groups etc.), they must be just single objects.
Text frames containing numbers of graphic pages to be placed must not belong to stories
(they must be just single text frames).
Will not be processed the text frames, which bounds are out of pasteboard.
Will not be processed the text frames are inside overset (invisible) text.
If it uses symbols of current page number inside text frames, then the style of numbering 
must be based on ordinar arabic figures.

Technical support: Sergey-Anosov@yandex.ru
https://sites.google.com/site/dtpscripting
*/
// название скрипта
var the_title = "ImposeMultiPage";
// версия скрипта
var the_version = "1.3";
// активный документ
var AD;
// имя активного документа
var AD_NAME = "";
// имя активного документа для вывода в диалоге
var AD_NAME_DIAL = "";
// путь активного документа
var AD_PATH = "";
// был ли сохренен документ
var AD_SAVED = false;
// файловая система
var WFS = false;
// активный разворот
var AS;
// имя активного разворота
var AS_NAME = "";
// является ли активный разворот мастером
var AS_IS_MASTER = false;
// массив текстовых фреймов для обработки
var FRAMES = new Array();
// массив текстовых фреймов (мастер элементов) для обработки
var MASTER_FRAMES = new Array();
// число текстовых фреймов в массиве для обработки
var N_FRAMES = 0;
// ведется ли обработка мастер элементов
var MASTER_ITEM = false;
// текущая страница при поиске и обработке текстовых фреймов
var CURRENT_PAGE;
// опции обрезки для PDF и AI
var CROP_OPTIONS_LIST_PDF_AI = 
[
	"Art", // 0
	"Crop", // 1
	"Trim", // 2
	"Bleed", // 3
	"Media", // 4
	"Visible layers", // 5
	"All layers" // 6
];
// начальное значение опций обрезки для PDF
var CROP_OPTIONS_PDF_INI = 4;
// начальное значение опций обрезки для AI
var CROP_OPTIONS_AI_INI = 3;
// опции обрезки для INDD
var CROP_OPTIONS_LIST_INDD = 
[
	"Page bounding box", // 0
	"Bleed bounding box", // 1
	"Slug bounding box" // 2
];
// начальное значение опций обрезки для INDD
var CROP_OPTIONS_INDD_INI = 1;
// опции фиттинга
var FIT_OPTION_LIST = 
[
	"Use Prefences settings", // 0
	"Fit content to frame", // 1
	"Fit content to frame proportionally", // 2
	"Fit frame to content", // 3
	"Fill frame proportionally", // 4
	"100% content scaling", // 5
	"Specified content scaling", //6
	"Specified content size" //7
];
// индекс для опции фиттинга "Specified content scaling"
var FIT_SCALING_INDEX = 6;
// индекс для опции фиттинга "Specified content size"
var FIT_SIZE_INDEX = 7;
// максимальное количество опций фиттинга
var FIT_OPTION_MAX = FIT_OPTION_LIST.length;
// начальное значение для опций фиттинга
var FIT_OPTION_INI = 5;
// текущее значение опции фиттинга
var FIT_CURRENT = FIT_OPTION_INI;
// значение опции фиттинга если выбраны 
// "Specified content scaling" или "Specified content size"
var FIT_SCALE_SIZE = FIT_OPTION_INI;
// список типов токументов
var TYPE_DOC_LIST =
[
	"PDF document", // 0
	"Adobe Illustrator document", // 1
	"Adobe InDesign document" // 2
];
var FILE_TYPE_TEXT = "";
// список типов расширений
var EXT_LIST = 
[
	".pdf", // 0
	".ai", // 1
	".indd" // 2
];
// список типов файлов для диалога
var TYPE_FILE_LIST = 
[
	"PDF", // 0
	"AI", // 1
	"INDD" // 2
];
// список типов маркирующих стилей
var MARK_STYLE_LIST = 
[
	"Object style (text frames)", // 0
	"Paragraph style (numbers)", // 1
	"Character style (numbers)" // 2
];
// массив объектных стилей документа
var obj_styles = new Array();
// количество объектных стилей документа
var N_obj_styles = 0;
// массив имен объектных стилей документа для вывода в диалоге
var OBJ_STYLE_LIST = new Array();
// начальное значение для объектных стилей в диалоге
var OBJ_STYLE_INI = 0;
// массив стилей параграфа документа
var par_styles = new Array();
// количество стилей параграфа документа
var N_par_styles = 0;
// массив имен стилей параграфа документа для вывода в диалоге
var PAR_STYLE_LIST = new Array();
// начальное значение для стилей параграфа в диалоге
var PAR_STYLE_INI = 0;
// массив стилей символов документа
var ch_styles = new Array();
// количество стилей символов документа
var N_ch_styles = 0;
// массив имен стилей символов документа для вывода в диалоге
var CH_STYLE_LIST = new Array();
// начальное значение для стилей символов в диалоге
var CH_STYLE_INI = 0;
// диапазон без выделения
var RANGE_LIST = 
[
	"Document (excludind master spreads)", // 0
	"Active spread"// 1
];
// начальное значение диапазона без выделения (активный разворот)
var RANGE_INI = 1;
// диапазон с выделением
var RANGE_LIST_SEL = 
[
	"Document (excludind master spreads)", // 0
	"Active spread", // 1
	"Selection"//2
];
// начальное значение диапазона с выделением
var RANGE_INI_SEL = 2;
// список режимов выполнения
var MODE_LIST =
[
	"Auto", // 0
	"Step-by-step"// 1
];
// тип документа для вывода в диалоге
var DOC_TYPE_DIAL = TYPE_DOC_LIST[0];
// расширение для вывода в диалоге
var EXT_DIAL = EXT_LIST[0];
// тип файла для вывода в диалоге
var FILE_TYPE_DIAL = TYPE_FILE_LIST[0];
// массив выделенных текстовых фреймов
var SEL_FRAMES = new Array();
// количество выделеных текстовых фреймов
var N_SEL_FRAMES = 0;
// текстовые фреймы в выделении
var TF_IN_SEL  = true;
// опции обрезки при помещении изображения
var CROP;
// текстовое значение для опции обрезки
var CROP_TEXT = "";
// индекс для опций обрезки из диалога
var CROP_INDEX;
// настройки фиттинга
var FIT;
// текстовое значение для настроек фиттинга
var FIT_TEXT = "";
// настройки центрирования
var CENTER;
// начальное значение настройки центрирования
var CENTER_INI = true;
// настройки прозрачности
var TRANSP;
// начальное значение настройки прозрачности
var TRANSP_INI = false;
// файл для помещения во фреймы
var FILE = new File();
// имя файла для помещения во фреймы
var FILE_NAME = "";
// имя файла для помещения во фреймы для вывода в диалоге
var FILE_NAME_DIAL = "";
// расширение файла для помещения во фреймы
var FILE_EXT = "";
// полный путь файла для помещения во фреймы
var FILE_PATH = "";
// массив страниц с некорректными номерами
var BAD_NUM = new Array();
// файл не был выбран
var FILE_NOT_CHOSEN = true;
// текст если файл не выбран
var NOT_CHOSEN_TXT = "is not chosen";
// обрабатывать ли текстовые фреймы на заблокированных слоях
var PROC_ON_LOCKED_LAYERS = false;
// обрабатывать ли заблокированные и/или невидимые текстовые фреймы
var PROC_LOCK = false;
// в документе присутствуют заблокированные или невидимые слои
var LAYERS_LOCKED = false;
// версия приложения новее CS4
var VER_CS5 = false;
// диапазон операции
var PROC_RANGE;
// тесктовое значение для диапазона операции
var PROC_RANGE_TEXT;
// обрабатывать ли мастер-элементы
var PROC_MASTER = false;
// режим выполнения
var PROC_MODE = 0;
// тип маркирующего стиля
var MARK_STYLE;
// имя маркирующего стиля
var STYLE_NAME;
// текстовое значение для имени маркирующего стиля
var STYLE_NAME_TEXT = "";
// в выделении заблокированные текстовые фреймы
var LOCKED_IN_SEL = false;
// массив разблокированных слоев
var LAYERS_UNLOCKED = new Array();
// массив сделанных видимыми слоев
var LAYERS_MADE_VISIBLE = new Array();
// обрабатывать только заблокированные текстовые фреймы
var PROC_LOCK_ONLY_TXT = "Process locked text frames";
// обрабатывать заблокированные и невидимые текстовые фреймы
var PROC_LOCK_INVISIBLE_TXT = "Process locked and invisible text frames";
// обрабатывать заблокированные
var PROC_LOCK_TXT;
// заголовок для предупреждения
var WARN_HEAD = "Warning:\n\n";
// начало текста предупреждения
var WARN_START = 
WARN_HEAD +
"Some of text frames are to be processed\n";
// окончание текста предупреждения
var WARN_END = 
"Current version "+the_version+" of the script\n"+
"сan not process that :(";
// выбран любой тип стиля для поиска
var ANY_STYLE_TYPE = false;
// текст для любого стиля
var ANY_STYLE_TEXT = "Any style type";
// индекс для стиля любого типа
var ANY_STYLE_INDEX = 4;
// единицы измерения в документе
var H_UNITS;// по горизонтали (для ширины)
var V_UNITS;// по вертикали (для высоты)
// начальные единицы измерения в документе
var H_UNITS_INI;// по горизонтали (для ширины)
var V_UNITS_INI;// по вертикали (для высоты)
// текстовые обозначения единиц измерения
var H_UNITS_TEXT = "";// по горизонтали
var V_UNITS_TEXT = "";// по вертикали
// список текстовых значений для единиц для версий CS3 и CS4
var UN_TEXT_LIST = 
[
	"mm",// 0
	"cm",// 1
	"pt",// 2
	"in",// 3
	"ind",// 4
	"pc",// 5
	"ci",// 6
	"ag"// 7
];
// список "разрешенных" единиц для версий CS3 и CS4
var UN_LIST =
[
	MeasurementUnits.MILLIMETERS,// 0
	MeasurementUnits.CENTIMETERS,// 1
	MeasurementUnits.POINTS,// 2
	MeasurementUnits.INCHES,// 3
	MeasurementUnits.INCHES_DECIMAL,// 4
	MeasurementUnits.PICAS,// 5
	MeasurementUnits.CICEROS,// 6
	MeasurementUnits.AGATES// 7
];
// начальные значения для единиц
var H_UNITS_DROP_INI = 0;// по горизонтали
var V_UNITS_DROP_INI = 0;// по вертикали
// единицы выбранные в диалоге
var H_UN;
var V_UN;
// менялись ли единицы измерения
var UN_CHANGED = false;
// неподходящие единицы измерения
var BAD_UNITS = false;
// результат текстового ввода
var INP_OK = true;
// начальные значения сдвигов для нечетных номеров
var DX_ODD_INI  = "0";// по горизонтали
var DY_ODD_INI  = "0";// по вертикали
// начальные значения сдвигов для четных номеров
var DX_EVEN_INI = "0";// по горизонтали
var DY_EVEN_INI = "0";// по вертикали
// сдвиги для нечетных номеров
var DX_ODD  = 0.;// по горизонтали
var DY_ODD  = 0.;// по вертикали
// сдвиги для четных номеров
var DX_EVEN = 0.;// по горизонтали
var DY_EVEN = 0.;// по вертикали
// начальные значения для масштабирования
var W_SCALE_INI = "100";// по горизонтали (для ширины)
var H_SCALE_INI = "100";// по вертикали (для высоты)
// значения для масштабирования в главном диалоге
var W_SCALE = 100.0;// по горизонтали (для ширины)
var H_SCALE = 100.0;// по вертикали (для высоты)
// значения для масштабирования в пошаговом диалоге
var W_SCALE_STEP = 100.0;// по горизонтали (для ширины)
var H_SCALE_STEP = 100.0;// по вертикали (для высоты)
// начальные значения для размеров
var W_SIZE_INI = "10";// по горизонтали (для ширины)
var H_SIZE_INI = "10";// по вертикали (для высоты)
// значения для размеров в главном диалоге
var W_SIZE = 10.0;// по горизонтали (для ширины)
var H_SIZE = 10.0;// по вертикали (для высоты)
// значения для размеров в пошаговом диалоге
var W_SIZE_STEP = 10.0;// по горизонтали (для ширины)
var H_SIZE_STEP = 10.0;// по вертикали (для высоты)
// возможность показать фрейм в пошаговом режиме
var CAN_SHOW = true;
// текстовые значения для кнопок вариантов действий в диалоге 
// для пошагового режима
var R_TEXT = "Reset data";
var S_TEXT = "Skip frame";
var Q_TEXT = "Quit operation";
var OK_TEXT = "Place into frame";
// число обработанных фреймов
var N_PROC = 0;
// число пропущенных фреймов
var N_SKIP = 0;
// число ошибок при помещении
var N_ER = 0;
// положение окна диалога для пошагового режима
var STEP_DIAL_LOCATION = [ false, undefined ];
// строка с выбранными настройками
var ALL_SET_STR = "";
// начальная страница диапазона
var START_PAGE = 0;
// конечная страница диапазона
var END_PAGE = 0;
// количество страниц в диапазоне
var N_PAGES = 0;
// пейстборд вверху
var PB_TOP = 0.;
// пейстборд слева
var PB_LEFT = 0.;
// некорректное положение текстового фрейма
// (верхняя или левая граница вне пейстборда)
// или он находится в переполненном тексте
var BAD_POS = false;
// текст для вывода сообщения о некорректном 
// положении текстового фрейма
var BAD_POS_TEXT = 
"Some text frames were excluded\n"+
"from the process because of\n" +
"their incorrect position (location)!";
// начальный текст для кнопки выбора файла
var BUTTON_CHOOSE_TEXT = "Choose file to place";
// текст для кнопки выбора файла (смена файла)
var BUTTON_CHANGE_TEXT = "Change file to place";
//
// вызов основного модуля
main();
//
// основной модуль
function main()
{
	// вызов подпрограммы начальной проверки
	CHECK();
	// вызов подпрограммы начального диалога
	DIALOG_INI();
};// end main
//
// подпрограмма начальной проверки 
function CHECK()
{
	// проверка наличия открытых документов
	if(app.documents.length == 0 )
	{
		the_exit("No documents are open!");
	};// if
	// определяем файловую систему
	if(File.fs == "Windows") WFS = true; 
	// активный документ
	AD = app.activeDocument;
	// имя активного документа
	AD_NAME = unescape(AD.name);
	// имя активного документа для вывода в диалоге
	AD_NAME_DIAL = SHORT_NAME( AD_NAME, WM(25, 25) );
	// был ли сохранен активный докмент
	AD_SAVED = AD.saved;
	// путь активного документа (если он был сохранен)
	if( AD_SAVED )
	{
		AD_PATH = unescape(File(AD.fullName).fullName);
	};// if
	// активный разворот
	AS = app.activeWindow.activeSpread; 
	// проверяем является ли активный разворот мастером
	AS_IS_MASTER = SPREAD_IS_MASTER( AS );
	// формируем массив имен объектных стилей
	obj_styles = AD.objectStyles;
	N_obj_styles = obj_styles.length;
	for( var i = 0; i < N_obj_styles; i++ )
	{
		OBJ_STYLE_LIST[i] = obj_styles[i].name;
	};// for
	// начальное значение списка имен объектных стилей
	OBJ_STYLE_INI = GET_INI_NAME_INDEX( OBJ_STYLE_LIST );
	// формируем массив имен стилей параграфа
	par_styles = AD.paragraphStyles;
	N_par_styles = par_styles.length;
	for( var i = 0; i < N_par_styles; i++ )
	{
		PAR_STYLE_LIST[i] = par_styles[i].name;
	};// for
	// начальное значение списка имен стилей пераграфа
	PAR_STYLE_INI = GET_INI_NAME_INDEX( PAR_STYLE_LIST );
	// формируем массив имен стилей символов
	ch_styles = AD.characterStyles;
	N_ch_styles = ch_styles.length;
	for( var i = 0; i < N_ch_styles; i++ )
	{
		CH_STYLE_LIST[i] = ch_styles[i].name;
	};// for
	// начальное значение списка имен стилей символов
	CH_STYLE_INI = GET_INI_NAME_INDEX( CH_STYLE_LIST );
	// проверка выделения
	// выделение
	var the_sel = app.activeDocument.selection;
	// длина выделения
	var SEL_LEN =  the_sel.length;
	// если длина выделения = 0 (ничего не выделено)
	if( SEL_LEN == 0 ) 
	{
		// в выделении НЕТ текстовых фреймов :)))
		TF_IN_SEL = false;
	}
	// если что-то выделено
	else
	{
		// ищем текстовые фреймы в выделении
		GET_TEXT_FRAMES ( the_sel, SEL_FRAMES, false );
		// кол-во текстовых фреймов в выделении
		N_SEL_FRAMES = SEL_FRAMES.length;
		// если кол-во текстовых фреймов в выделении = 0
		if( N_SEL_FRAMES == 0 ) 
		{
			// в выделении НЕТ текстовых фреймов :)))
			TF_IN_SEL = false;
		};// if
	};// if-else
	// если в выделении есть текстовые фреймы
	if( TF_IN_SEL )
	{
		// проверяем есть ли заблокированные текстовые фреймы в выделении
		for( var i=0; i < N_SEL_FRAMES; i++ )
		{
			// если текстовый фрейм в выделении заблокирован
			if( SEL_FRAMES[i].locked )
			{
				// в выделении есть заблокированные текстовые фреймы
				LOCKED_IN_SEL = true;
				break;
			};// if
		};// for
		// добавляем количество выделенных фреймов в пункт диалога для выделения
		RANGE_LIST_SEL[2] = RANGE_LIST_SEL[2] + N_TF_STR( N_SEL_FRAMES );
	};// if
	// проверяем есть ли невидимые или заблокированные слои в документе
	for( var i=0; i< AD.layers.length; i++)
	{
		// если слой документа заблокирован или невидимый
		if( (!AD.layers[i].visible) || ( AD.layers[i].locked ) )
		{
			// в документе присутствуют заблокированные и/или невидимые слои
			LAYERS_LOCKED = true;
			break;
		};// if
	};// for
	// проверяем версию индизайна
	var the_app_version = parseInt(app.version);
	// если версия индизайна новее CS4
	if( the_app_version >= 7 ) VER_CS5 = true;
	// получаем имя активного разворота
	AS_NAME = GET_SPR_NAME( AS );
	// добавляем имя активного разворота  и страниц к выбору диапазона
	var AS_name = " (" + AS_NAME + " "+GET_PAGES_NAME(AS)+")";
	// для диапазона без выделения
	RANGE_LIST[1] = RANGE_LIST[1] + AS_name ;
	// для диапазона с выделением
	RANGE_LIST_SEL[1] = RANGE_LIST_SEL[1] + AS_name;  
	// получаем единицы измерения в документе и заодно запоминаем старые значения
	H_UNITS_INI = H_UNITS = AD.viewPreferences.horizontalMeasurementUnits;
	V_UNITS_INI = V_UNITS = AD.viewPreferences.verticalMeasurementUnits;
	// получаем текстовые обозначения единиц измерения
	H_UNITS_TEXT = GET_RULER_UNITS_TEXT( H_UNITS_INI );
	V_UNITS_TEXT = GET_RULER_UNITS_TEXT(V_UNITS_INI );
	// настройки для пейстборда
	// для CS5 и новее
	if( VER_CS5 )
	{
		// пейстборд вверху
		PB_TOP = AD.pasteboardPreferences.pasteboardMargins[1];
		// пейстборд слева
		PB_LEFT = AD.pasteboardPreferences.pasteboardMargins[0];
		if( PB_LEFT <= 0. ) PB_LEFT = AD.documentPreferences.pageWidth;
	}
	// для CS4 и более ранние версии
	else
	{
		// пейстборд вверху
		PB_TOP = AD.pasteboardPreferences.minimumSpaceAboveAndBelow;
		// пейстборд слева
		PB_LEFT = AD.documentPreferences.pageWidth;
		// единицы
		var H_UNITS_CHECK = UNITS_OK( H_UNITS );
		var H_UNITS_OK = H_UNITS_CHECK[ 0 ];
		H_UNITS_DROP_INI = H_UNITS_CHECK[ 1 ];
		var V_UNITS_CHECK = UNITS_OK( V_UNITS );
		var V_UNITS_OK = V_UNITS_CHECK[ 0 ];
		V_UNITS_DROP_INI = V_UNITS_CHECK[ 1 ];
		// проверяем соответствие единиц измерения документа "разрешенному" списку
		if( !H_UNITS_OK || !V_UNITS_OK )
		{
			// если хотя бы одни из единиц не соответсвуют 
			// считаем что требуется ввод единиц в диалоге
			BAD_UNITS = true;
			// получаем текстовые обозначения единиц измерения
			H_UNITS_TEXT = UN_TEXT_LIST[H_UNITS_DROP_INI];
			V_UNITS_TEXT = UN_TEXT_LIST[V_UNITS_DROP_INI];
		};// if
	};// if-else
	return;
};// end CHECK
//
// подпрограмма проверки соответствия единиц измерения
// "разрешенному" списку
function UNITS_OK( the_units )
{
	for( var i = 0; i < UN_LIST.length; i++ )
	{
		if( the_units == UN_LIST[i] ) return [ true, i ];
	};// for
	return [false, 0 ];	
};// end UNITS_OK
//
// подпрограмма восстановления начальных единиц измерения
function RESTORE_UNITS()
{
	AD.viewPreferences.horizontalMeasurementUnits = H_UNITS_INI;
	AD.viewPreferences.verticalMeasurementUnits = V_UNITS_INI;
	return;
};// end RESTORE_UNITS
//
// подпрограмма формирования строки с количеством фреймов
// N - количество фреймов
function N_TF_STR( N )
{
	var tf_tfs = "text frame";
	if( N > 1 ) tf_tfs = tf_tfs + "s";
	return ( " (" + N.toString() + " "+tf_tfs+")" );
};// end N_TF_STR
//
// подпрограмма определения текста единиц для линеек документа
function GET_RULER_UNITS_TEXT(the_units) 
{
	switch ( the_units ) 
	{
		case (2054188905): return "pt";// points
		case (2054187363): return "pc";// picas
		case (2053729891): return "in";// inches
		case (2053729892): return "ind";// inches decimal
		case (2053991795): return "mm";// millimeters
		case (2053336435): return "cm";// centimeters
		case (2053335395): return "ci";// ciceros
		case (1131639917): return "pt";// custom
		case (2051106676): return "ag";// agates
		case (2054187384): return "px";// pixels
		default: return "??";
	};// switch
	return "??";
};// end GET_RULER_UNITS_TEXT;
//
// подпрограмма получения начального значения для 
// списка имен стилей
function GET_INI_NAME_INDEX( list )
{
	try
	{
		// цикл по именам в списке
		for( var i=0; i < list.length; i++ )
		{
			// текущее имя
			var L = list[i];
			// если это не имя по умолчанию
			// у имен по умолчанию обычно в начале и в конце квадратные скобки
			if( (L[0] != "[") && (L[ L.length-1] != "]" ) )
			{
				// считаем его начальным для диалога
				return i;
			};// if
		};// for i
	} catch ( error ) {};
	// если все стили по умолчанию тогда самое первое имя
	return 0;
};// end GET_INI_NAME_INDEX
//
// подпрограмма поиска текстового фрейма в массиве объектов
function GET_TEXT_FRAMES ( arr_in, arr_out, check )
// arr_in - входящий массив объектов
// arr_out - выходящий массив текстовых фреймов
// check - проверять ли соответствие параметров
{
	// подпрограмма проверки находится ли объект уже в выходном массиве
	// obj - проверяемый объект
	// the_arr - массив, принадлежность к которому проверяется
	function IS_IN_ARR( obj, the_arr )
	{
		// ID проверяемого объекта
		var the_obj_id = obj.id;
		// цикл по объектам массива
		for( var i = 0; i < the_arr.length; i++ )
		{
			// если совпали ID значит объект уже в массиве
			if( the_obj_id == the_arr[i].id ) return true;
		};// for
		// если не было совпадений ID значит объект не в массиве
		return false;
	};// end IS_IN_ARR
	//
	// подпрограмма проверки позиции текстового фрейма
	// В - границы текстового фрейма
	function CHECK_POSITION( B )
	{
		// границы объекта
		try
		{
			// верх объекта
			var top = B[0];
			// лево объекта
			var left = B[1];
			// низ объекта
			var bottom = B[2];
			// право объекта
			var right = B[3];
			// проверка по верхней границе и левой границе
			if( (top <= -1.* PB_TOP) || (left <= -1.* PB_LEFT) )
			{
				// если верхняя граница меньше пейстборда вверху 
				// или левая граница меньше пейстборда слева
				// (объект вверху или слева вышел за пейстборд)
				// считаем положение текстового фрейма некорректным
				return false;
			};// if
		} catch ( error ) {};
		return true;
	};// end CHECK_POSITION
	//
	// цикл по входному массиву
	for( var i=0; i < arr_in.length; i++ )
	{
		// текущий объект
		var the_obj = arr_in[i];
		// вид объекта
		var the_name = the_obj.constructor.name;
		// если объект является текстовым фреймом
		if( the_name == "TextFrame" )
		{
			// если текстового фрейма еще нет в выходном массиве
			if( !IS_IN_ARR( the_obj, arr_out ) )
			{
				// помещать ли в результате фрейм в выходной массив
				// начальное присвоение не помещать, если выполнятся условия
				// тогда помещаем
				var arr_out_push = false;
				//
				// если задана проверка параметров
				if( check )
				{
					// проверяем можно ли получить границы фрейма
					var the_obj_bounds = GBOUNDS( the_obj );
					// если нельзя получить границы текстового фрейма
					if( !the_obj_bounds[0] )
					{
						// то скорее всего текстовый фрейм в переполненном тексте и пропускаем его
						BAD_POS = true;
						continue;
					}
					// если границы можно получить проверяем положение фрейма
					else
					{
						// если неправильное положение фрейма пропускаем его
						if( !CHECK_POSITION( the_obj_bounds[1] ) )
						{
							BAD_POS = true;
							continue;
						};// if
					};// if-else
					// если не выбран "любой" тип стиля для поиска
					if( !ANY_STYLE_TYPE)
					{
						// проверяем соответствие по стилю
						// сначала считаем, что соответствия нет
						// (обратное надо будет доказать делом :)))
						var style_ok = false;
						// если поиск по объектному стилю
						if( MARK_STYLE == 0 )
						{
							// объектный стиль текстового фрейма
							var the_obj_style = the_obj.appliedObjectStyle;
							// соответствие имени объектного стиля текстового фрейма и заданного 
							if( the_obj_style.name == STYLE_NAME ) 
							{
								style_ok = true;
							};// if
						};// if MARK_STYLE == 0
						// если поиск по стилю параграфа
						if( MARK_STYLE == 1 )
						{
							// цикл по символам в текстовом фрейме
							for( var j = 0; j < the_obj.characters.length; j++ )
							{
								// соответствие стиля параграфа у символа и заданного стиля параграфа 
								if( the_obj.characters[j].appliedParagraphStyle.name == STYLE_NAME )
								{
									style_ok = true;
									break;// for j
								};// if
							};// for j
						};// if MARK_STYLE == 1
						// если поиск по стилю символа
						if( MARK_STYLE == 2 )
						{
							// цикл по символам в текстовом фрейме
							for( var j = 0; j < the_obj.characters.length; j++ )
							{
								// соответствие символьного стиля у символа и заданного символьного стиля 
								if( the_obj.characters[j].appliedCharacterStyle.name == STYLE_NAME )
								{
									style_ok = true;
									break;// for j
								};// if
							};// for j
						};// if MARK_STYLE == 2
					}
					// если выбран "любой" тип стиля
					else
					{
						// считаем что текстовый фрейм уже найден 
						// по заданным условиям, стиль не проверялся и подходит любой :)))
						style_ok = true;
					};// if-else
					//
					// результат проверки по стилю: 
					// если нет соответствия тогда в начало цикла (дальше не проверяем)
					if( !style_ok ) continue;
					// 
					// проверка заблокированных и невидимых текстовых фреймов
					var lock_ok = true;
					// если задано НЕ обрабатывать заблокированные и невидимые текстовые фреймы
					if( !PROC_LOCK )
					{
						// если текстовый фрейм заблокирован
						if( the_obj.locked )
						{
							// не разрешаем помещать заблокированный текстовый фрейм
							// в выходной массив
							lock_ok = false;
						};// if
						// проверка видимости текстового фрейма
						// если текстовый фрейм вообще имеет свойство видимости (в новых вырсиях индизайна)
						if( the_obj.hasOwnProperty ("visible") )
						{
							// если текстовый фрейм невидимый
							if( !the_obj.visible )
							{
								// не разрешаем помещать невидимый текстовый фрейм
								// в выходной массив
								lock_ok = false;
							};// if
						};// if
					};// if  !PROC_LOCK
					//
					// результат проверки для заблокированных и невидимых текстовых фреймов
					// если при НЕ заданной обработке заблокированных и невидимых текстовых фреймов
					// текстовый фрейм заблокированный или невидим тогда в начало цикла (дальше не проверяем )
					if( !lock_ok ) continue;
					//
					// проверка для слоя на котором находится текстовый фрейм
					// (слой невидимый или заблокированный)
					var layer_ok = true;
					// если задано НЕ обрабатывать текстовые фреймы на невидимых или заблокированных слоях
					if( !PROC_ON_LOCKED_LAYERS )
					{
						// получаем слой на котором расположен фрейм
						var the_layer = GET_ITEM_LAYER( the_obj );
						// если не удалось получить слой фрейма пропускаем этот фрейм
						// вероятно этот фрейм плохой :)))
						if( the_layer == undefined ) continue;
						// если слой заблокирован
						if( the_layer.locked )
						{
							layer_ok = false;
						};// if
						// если слой невидимый
						if( !the_layer.visible )
						{
							layer_ok = false;
						};// if
					};// if !PROC_ON_LOCKED_LAYERS
					//
					// результат проверки для слоя на котором находится текстовый фрейм
					// если при НЕ заданной обработке заблокированных или невидимых слоев
					// этот слой заблокирован или невидимый тогда в начало цикла (дальше не проверяем)
					if( !layer_ok ) continue;
					//
					// проверка на принадлежность фрейма к цепочкам
					// если больше 1 текстового фрейма в цепочке сразу выход
					CHECK_STORY( the_obj );
					//
					// если поиск среди мастер элементов
					if( MASTER_ITEM )
					{
						// проверка является ли текстовый фрейм отдельным объектом
						// (не входит в группы и т.п.)
						CHECK_MASTER_ITEM( the_obj );
					};// if
					//
					// итоговый вывод по всем параметрам 
					// (помещать текстовый фрейм в выходной массив или нет)
					// если дошли до этого места и все ОК тогда считаем что текстовый фрейм
					// можно поместить в выходной массив
					arr_out_push = true;
				}
				// если НЕ задана проверка параметров 
				else
				{
					// разрешаем поместить текстовый фрейм в выходной массив сразу
					arr_out_push = true;
				};// if-else check
				// помещаем текстовый фрейм в выходной массив если задано помещение
				if( arr_out_push ) 
				{
					// текстовый фрейм
					arr_out.push( the_obj );
				};// if
			};// if !IS_IN_ARR
		}
		// если объект не текстовый фрейм
		else
		{
			try
			{
				// то пытаемся в нем найти текстовые фреймы
				// формируем массив всех составляющих объектов (вдруг это группа и т.п.)
				var pg_items_arr = the_obj.allPageItems;
				// для этого подпрограмма вызывает себя же
				// как массив для поиска передается массив всех составляющих объектов
				GET_TEXT_FRAMES ( pg_items_arr, arr_out, check );
			} catch ( error ) {};
		};// if-else (the_name == "TextFrame")
	};// for i
	return;
};// end GET_TEXT_FRAMES
//
// подпрограмма получения слоя, на котором находится фрейм
// F - фрейм, для которого определяеся слой
function GET_ITEM_LAYER( F )
{
	// слой объекта
	var L ;
	// количество шагов поиска
	var N = 0;
	// начальное присвоение объекта
	var the_obj = F;
	do
	{
		N++;
		// выход если превышено адекватное число шагов
		if( N > 5000 ) break;
		try
		{
			// собственно получаем слой объекта
			L = the_obj.itemLayer;
			break;
		} 
		// если ошибка считаем, что фрейм находится
		// в переполнении в другом фрейме
		catch ( error )
		{
			try
			{
				// переприсваиваем объект как родительский объект
				the_obj = the_obj.parent;
				// получаем имя родительского объекта
				the_obj_name = the_obj.constructor.name;
				// если родительский объект цепочка фреймов
				if( the_obj_name == "Story" )
				{
					// учитываем разные версии индизайна
					// для новых версий
					if( the_obj.hasOwnProperty ("textContainers") )
					{
						// количество текстовых фреймов в цепочке
						var f_num = the_obj.textContainers.length;
						// объект по которому определяется слой фрейма
						the_obj = the_obj.textContainers[ f_num - 1 ];
					}
					// для старых версий
					else
					{
						// количество текстовых фреймов в цепочке
						var f_num = the_obj.textFrames.length;
						// объект по которому определяется слой фрейма
						the_obj = the_obj.textFrames[ f_num - 1 ];
					};// if-else
					continue;
				};// if
			} catch ( error ) {};
		};// try-catch
	// заканчиваем поиск если дошли до уровня приложения
	} while ( the_obj_name != "Application");
	return L;
};// end GET_ITEM_LAYER
//
// подпрограмма деактивации / активации списка
function DROP_ACTIVE_DEACTIVE( the_drop, act ) 
{
	// дополнительный пункт НЕАКТИВНО (N/A)
	var NA;
	// если деактивируем 
	if( !act ) 
	{
		try
		{
			// если он уже деактивирован возврат
			if( !the_drop.enabled ) return;
			// добавляем пункт НЕАКТИВНО
			NA = the_drop.add('item', "N/A");
			// ставим выделение на этом пункте
			NA.selected = true;
			// собственно деактивируем
			the_drop.enabled = false;
		} catch ( error ) {};
	} 
	else 
	{
		try 
		{
			// если он уже активирован возврат
			if( the_drop.enabled ) return;
			// удаляем добавленный пункт НЕАКТИВНО
			the_drop.remove(  the_drop.items[the_drop.items.length-1]  );
			// ставим выделение на 1 пункте
			the_drop.items[0].selected = true;
			// собственно активируем
			the_drop.enabled = true;
		} catch ( error ) {};
	};// if
	return;
};// end DROP_ACTIVE_DEACTIVE
//
// подпрограмма изменения вариантов выбора
// в дропдауне
function REFILL_DROP( d, arr, ini, sep )
{
	// удаляем все старые варианты
	for( var i=d.items.length; i >= 0; i-- )
	{
		try
		{
			d.remove(  d.items[i]  );
		} catch ( error ) {};
	};// for
	// заполняем новыми вариантами
	for( var i=0; i < arr.length; i++ ) 
	{
		d.add( 'item', arr[i] );
	};// for
	// если задан текст для варианта после разделителя
	if( sep !="" )
	{
		// создаем разделитель
		d.add( 'separator' );
		// добавляем пункт после разделителя
		d.add( 'item', sep );
	};//if
	// присваиваем начальный вариант
	d.selection = ini;
	return;
};// end REFILL_DROP
//
// подпрограмма назначения ширины дропдауна
function DDW( d, w )
{
	d.minimumSize.width =
	d.maximumSize.width = w;
	return;
};// end DDW
//
// подпрограмма окончания скрипта
function the_exit( t )
{
	// если есть текст сообщения
	if( t != "" )
	{ 
		// выводим это сообщение
		alert( t );
	}
	// если нет текста сообщения
	else
	{
		// выводим отчет
		REPORT();
	};// if-else
	// собственно выход
	the_quit();
};// end the_exit
//
// подпрограмма выхода без сообщений
function the_quit()
{
	if( UN_CHANGED ) RESTORE_UNITS();
	exit();
};// end the_quit
//
// подпрограмма сокращения имени
// txt - строка на входе
// N - количество символов в выходной строке
function SHORT_NAME(txt, N) 
{
	if ( txt.length > N ) txt = txt.substr(0, N) + "...";
	return txt;
};// end SHORT_NAME
//
// подпрограмма получения статуса разворота
// мастер или обычный разворот
function SPREAD_IS_MASTER( the_spr )
{
	try
	{
		if( the_spr.constructor.name == "MasterSpread" ) return true;
	} catch (error) {};
	return false
};// SPREAD_IS_MASTER
//
// подпрограмма получения имени или номера активного разворота
function GET_SPR_NAME( the_spr )
{
	var the_name = "";
	// если мастер
	if( SPREAD_IS_MASTER( the_spr ) )
	{
		// даем имя мастер разворота
		the_name = the_spr.name;
	}
	// если обычный разворот даем номер
	else
	{
		the_name = the_spr.index + 1;
		the_name = "#" + the_name;
	};// if-else
	return the_name;
};// end GET_SPR_NAME
//
// подпрограмма получения имен для полос разворота
function GET_PAGES_NAME( the_spr )
{
	var P_NAMES = "";
	var P_TEXT = "Page";
	var N_P = the_spr.pages.length;
	// если страниц больше 1 то учитываем множественное число
	if( N_P > 1 ) P_TEXT = P_TEXT +"s";
	P_TEXT = P_TEXT +": ";
	// цикл по страницам разворота
	for(var i = 0; i < N_P; i++)
	{
		// имя текущей страницы
		var the_name = the_spr.pages[i].name;
		// если только 1 старница на развороте
		if( N_P == 1) 
		{
			P_NAMES = P_NAMES + the_name;
			// сразу выход из цикла чтобы не добавлять дефис
			break;
		};// if
		// для первой страницы
		if( i == 0)
		{
			P_NAMES = P_NAMES + the_name;
			// сразу в начало цикла чтобы не добавлять дефис
			continue;
		};// if
		// добавляем дефис
		P_NAMES = P_NAMES+"-"+the_name;
	};// for
	// собственно строка сформирована
	P_NAMES = P_TEXT + P_NAMES;
	return P_NAMES;
};// end GET_PAGES_NAME
//
// подпрограмма создания чекбокса
function IN_CB( where, name, len, ini)
{
	var cb = where.add('checkbox');
	cb.text = name;
	cb.value = ini;
	cb.characters = len;
	return cb;
};// end IN_CB
//
// подпрограмма создания дропдауна
function IN_DATA_DROP( where, name, data_arr, extra, data_ini, dd_text_len, dd_len )
{
	var the_line = where.add('group');
	// создаем строку
	the_line.orientation = 'row';
	the_line.alignChildren = 'left';
	// название
	var the_name = the_line.add('statictext');
	the_name.text = name;
	the_name.characters = dd_text_len;
	// создаем дропдаун
	var dr = the_line.add('dropdownlist');
	for( var i=0; i < data_arr.length; i++ )
	{
		dr.add( 'item', data_arr[i] );
	};// for
	// добавляем разделитель если он задан
	if( extra != "" )
	{
		dr.add( 'separator' );
		dr.add( 'item', extra );
	};// if
	DDW( dr, dd_len );
	dr.selection = data_ini;
	return dr;
};// end IN_DATA_DROP
//
// подпрограмма создания текстовой строки в диалоге
function DTEXT( where, txt )
{
	var the_text = where.add('statictext');
	the_text.text = txt;
	return the_text;	
};// end DTEXT
//
// подпрограмма создания кнопки
function BUTTON( where, b_name, text_ini )
{
	var g = where.add('group');
	g.orientation = 'row';
	g.alignChildren = 'left';
	var b = g.add('button');
	b.text = b_name;
	var t = "";
	// текст справа
	if( text_ini != "" )
	{
		var t = g.add('statictext');
		// размер поля для текста справа
		t.characters = WM( 17, 15 );
		t.text = text_ini;
	};// if
	return [b, t];
};// end BUTTON
//
// подпрограмма создания блока для ввода размеров
function DIM_IN (where, b_name, b_name_len, name_1, ini_1, un_1, name_2, ini_2, un_2 )
{
	// создаем строку для всех объектов
	var the_line = where.add('group');
	the_line.orientation = 'row';
	the_line.alignChildren = 'left';
	// название
	if( the_name != "" )
	{
		var the_name = the_line.add('statictext');
		the_name.text = b_name;
		the_name.characters = b_name_len;
	};// if
	// длина поля ввода
	var data_len = WM( 48, 46 );
	// первое поле
	var field_1 = NUM_FIELD( the_line, name_1, ini_1, data_len, un_1 );
	var f_1 = field_1[ 0 ];
	var t_1 = field_1[ 1 ];
	// второе поле
	var field_2 = NUM_FIELD( the_line, name_2, ini_2, data_len, un_2 );
	var f_2 = field_2[ 0 ];
	var t_2 = field_2[ 1 ];
	return [ f_1, f_2, t_1, t_2 ];
};// end DIM_IN
//
// подпрограмма ввода размера или масштабирования 
// для помещаемого изображения
function SIZE_IN_DIAL( b_name, type )
{
	var d = new Window('dialog');
	d.text = the_title + " v." +the_version; // заголовок диалога
	d.orientation = 'column';
	d.alignChildren = 'center';
	var mp = d.add('panel');
	b_name = b_name +":";
	// если тип данных проценты
	if( type == "%" ) 
	{
		var b_name_len = WM( 20, 20 );
		var ini_1 = W_SCALE_INI;
		var un_1 = "%";
		var ini_2 = H_SCALE_INI;
		var un_2 = "%";
	}
	// если тип данных размер
	else
	{
		var b_name_len = WM( 18, 18 );
		var ini_1 = W_SIZE_INI;
		var un_1 = H_UNITS_TEXT;
		var ini_2 = H_SIZE_INI;
		var un_2 = V_UNITS_TEXT;
	};//if-else
	var S = DIM_IN ( mp , b_name, b_name_len, "Width:", ini_1, un_1, "Height:", ini_2, un_2 );
	//
	// создание панели кнопок (кнопка ОК, кнопка Cancel)
	var okPanel = d.add('group');
	okPanel.orientation = 'row';
	// кнопка ОК
	var okBtn = okPanel.add('button', undefined, 'OK');
	// кнопка ОК неактивна, пока не выбран файл для помещения
	// кнопка Cancel
	var cancelBtn = okPanel.add('button', undefined, 'Cancel');
	// выводим основной диалог на экран
	var BUTTON = d.show();
	// нажимался ли ОК в диалоге
	var OK_PRESSED = false;
	// если нажато ОК
	if( BUTTON == 1 )
	{
		OK_PRESSED = true;
		// проверка ввода для процентов
		if( type == "%" )
		{
			if
			( 
				// если проценты меньше 0 или больше 100
				(TEXT_TO_DIGIT (S[0].text) > 100.) || (TEXT_TO_DIGIT (S[1].text) > 100.) ||
				(TEXT_TO_DIGIT (S[0].text) <= 0.) || (TEXT_TO_DIGIT (S[1].text) <= 0.)
			)
			{
				alert("Bad scaling input!");
				OK_PRESSED = false;
			};// if
		}
		// проверка ввода для размера
		else
		{
			// если размер меньше 0
			if( (TEXT_TO_DIGIT (S[0].text) <= 0.) || (TEXT_TO_DIGIT (S[1].text) <= 0.) )
			{
				alert("Bad size input!");
				OK_PRESSED = false;
			};// if
		};// if
	};// if-else
	// возвращаем: нажималась ли кнопка ОК и числа из полей ввода
	return [ OK_PRESSED, parseFloat( (TEXT_TO_DIGIT( S[0].text).toString() ) ), parseFloat( (TEXT_TO_DIGIT( S[1].text).toString() ) ) ];
};// end SIZE_IN_DIAL
//
// подпрограмма создания поля текстового ввода для числа
function NUM_FIELD( where, name, ini_data, data_len, un_text )
{
	// создаем строку для всех объектов
	var the_line = where.add('group');
	the_line.orientation = 'row';
	the_line.alignChildren = 'left';
	// название
	var the_name = the_line.add('statictext');
	the_name.text = name;
	// поле ввода
	var data_field = the_line.add('edittext');
	data_field.text = ini_data;
	data_field.preferredSize.width = data_len;
	data_field.onChanging = data_field_onChanging;
	//
	// подпрограмма реакции на процесс ввода числа в поле
	function data_field_onChanging()
	{
		digit_on_Changing( data_field, ini_data, ini_data );
		return;
	};// end data_field_onChanging
	// единицы после поля ввода (если они заданы)
	var un = undefined;
	if( un_text != "" )
	{
		un = the_line.add('statictext');
		un.text = un_text;
		un.characters = WM( 3, 3 );
	};// if
	return [ data_field, un];
};// end NUM_FIELD
//
// обработка поля ввода в процессе ввода
function digit_on_Changing(et, def, emp) 
{
	var s =et.text;
	if( s == "") 
	{
		et.text = emp;
		return;
	};
	if( !INP_OK ) 
	{
		et.text = def;
		INP_OK = true;
		return;
	}
	var d;
	var BNP = false;
	var N_C = 0;
	var N_P = 0;
	var N_M = 0;
	for( var i=0; i < s.length; i++ ) 
	{
		var si = s[i];
		if( si == "." || si == "," ) N_C = N_C +1;
		if( si == "-" ) N_M = N_M +1;
		if( si == "+" ) N_P = N_P +1;
		if
			( 
			((si != "-" && si != "+" && si != "." && si != "," && NO_DIGIT_SYMBOL(si)) || N_C>1 ) ||
			((N_P + N_M)>1) ||
			( si == "-"  && i !=0 ) ||
			( si == "+"  && i !=0 )
			) 
		{
			BNP = true;
			et.active = true;
			INP_OK = false;
			alert("Bad number input!");
			et.text = def;
			return;
		};// if
	};// for
	return;
};// end digit_on_Changing
//
// подпрограмма определения является ли символ цифрой
function NO_DIGIT_SYMBOL( s ) 
{
	try
	{
		if( s!= "0" && s!= "1" && s!= "2" && s!= "3" && s!= "4" && 
			 s!= "5" && s!= "6" && s!= "7" && s!= "8" && s!= "9") return true;
	} catch( error ) {};
	return false;
};// end NO_DIGIT_SYMBOL
//
// функция чтения текстового ввода и перевода в число
function TEXT_TO_DIGIT ( txt ) 
{
	var s = "";
	// если пусто то ноль
	if( txt == "" ) s = "0";
	if( txt == "+" ) s = "0";
	if( txt == "-" ) s = "0";
	var only_space = true;
	for( var i=0; i < txt.length; i++ ) 
	{
		if( txt[i] != " " ) only_space = false;
		if( txt[i] == ",") s = s + "." else s = s + txt[i];
	};// for
	// если только пробелы то ноль
	if( only_space ) s = "0";
	if( s == ".")  s = "0";
	if( s == "-")  s = "0";
	var s1 = "";
	for( var i=0; i < s.length; i++ ) 
	{
		if( s[i] == " " ) continue;
		s1 = s1 + s[i];
	};// for
	var d = parseFloat( s1 );
	return d;
};// end TEXT_TO_DIGIT
//
// подпрограмма ввода опций помещения страницы файла
// (для основного и пошагового диалога)
function PLACEMENT_SETTINGS
			( 
			where, // где (в каком диалоге) выводится блок
			main, // является ли диалог главным
			PS_CROP_LIST_INI,  // список опций
			PS_CROP_OPTION_INI, // начальная опция
			PS_TRANSP_INI, // начальная опция для прозрачности
			PS_FIT_OPTION_LIST, // список опций для фиттинга
			PS_FIT_OPTION_INI, // начальное значение для фиттинга
			PS_CENTER_INI // начальное значение для центрирования
			)
{
	// длина полей для дропдаунов
	var drop_data_len = WM( 130, 130 );
	// длина полей для имен дропдаунов
	var name_len = WM( 18, 18 );
	// длина текста для чекбоксов
	var CB_TEXT_LEN = WM( 35, 35 );
	// выбор опций обрезки
	var CROP = IN_DATA_DROP( where, "Crop options: ", PS_CROP_LIST_INI, "", PS_CROP_OPTION_INI, name_len, drop_data_len );
	// выбор опции прозрачности фона
	var TRANSP = IN_CB( where, "Transparent background", CB_TEXT_LEN, PS_TRANSP_INI);
	// выбор опций фиттинга
	// длина имени для дропдауна фиттинга
	var fit_name_len = WM( 6, 6 );
	// длина поля ввода для дропдауна фиттинга
	var fit_dd_len = WM( 214, 237 );
	var FIT = IN_DATA_DROP( where, "Fitting:", PS_FIT_OPTION_LIST, "", PS_FIT_OPTION_INI, fit_name_len, fit_dd_len );
	FIT.onChange = FIT_onChange;
	// выбор опции центрирования
	var CENTER = IN_CB( where, "Center content in frame", CB_TEXT_LEN, PS_CENTER_INI);
	//
	// подпрограмма реакции на смену опции фиттинга
	function FIT_onChange()
	{
		// подпрограмма для контроля и коррекции
		// состава списка опций фиттинга
		function RESET_FIT_OPTION_LIST()
		{
			// если длина списка больше максимальной
			// (то есть в списке присутствует разделитель и дополнительный пункт)
			if( PS_FIT_OPTION_LIST.length > FIT_OPTION_MAX )
			{
				// сокращаем список до стандартного размера
				// временный массив для временного хранения
				var LIST = PS_FIT_OPTION_LIST;
				// очищаем старый список
				PS_FIT_OPTION_LIST = null;
				for( i=0; i < FIT_OPTION_MAX; i++ ) 
				{
					// перезаполняем из временного массива
					PS_FIT_OPTION_LIST[ i ] = LIST[ i ];
				};// for
			};// if
			return;
		};// end RESET_FIT_OPTION_LIST
		// индекс опции
		var N = FIT.selection.index;
		// если совпал с текущим значением (т.е. не изменился)
		// тогда сразу выход
		if( N == FIT_CURRENT ) return;
		// запоминаем текущее значение опции
		FIT_CURRENT = N;
		// проверяем длину списка
		RESET_FIT_OPTION_LIST();
		// если выбран ввод масштаба или размера
		if( (N == FIT_SCALING_INDEX) || (N == FIT_SIZE_INDEX) )
		{
			// получаем конкрентное значение выбора
			FIT_SCALE_SIZE = N;
			// текстовая строка для диалога ввода значений
			var DIM_TEXT = PS_FIT_OPTION_LIST[ N ];
			// индекс для выделения пункта со значениями
			var N_SPEC = FIT_SCALING_INDEX+3;
			// если выбрано масштабирование
			if( N == FIT_SCALING_INDEX )
			{
				var N_TO_FIXED = 3;
				// вызываем диалог ввода значений для масштабирования
				var SIZE_DATA = SIZE_IN_DIAL( DIM_TEXT, "%" );
				try
				{
					var SIZE_IN_DIAL_OK = SIZE_DATA[0];
					var W = SIZE_DATA[1];
					var H = SIZE_DATA[2];
				} catch ( error ){};
				// если в диалоге ввода масштаба нажато ОК
				if( SIZE_IN_DIAL_OK )
				{
					// строка для дропдауна с масштабированием
					DIM_TEXT = 
					"W: "+ W.toFixed(N_TO_FIXED).toString() + "% "+
					"H: "+H.toFixed(N_TO_FIXED).toString()+ "%";
					// значения для масштабирования
					if( main )
					{
						W_SCALE = W;
						H_SCALE = H;
					}
					else
					{
						W_SCALE_STEP = W;
						H_SCALE_STEP = H;
					};// if-else
					// новые начальные значения для масштабирования
					// если ввод в главном диалоге
					if( main )
					{
						W_SCALE_INI = W.toString();
						H_SCALE_INI = H.toString();
					};// if
				}
				// если в диалоге был нажат Cancel 
				// или был некорректный ввод масштабирования
				else
				{
					// присваиваем выбору опций фиттинга значение по умолчанию
					FIT.selection = FIT_OPTION_INI;
					return;
				};// if-else
			};// if N == FIT_SCALING_INDEX
			// если выбран размер
			if( N == FIT_SIZE_INDEX )
			{
				var N_TO_FIXED = 4;
				// вызываем диалог ввода значений для размера
				var SIZE_DATA = SIZE_IN_DIAL( DIM_TEXT, "size" );
				try
				{
					var SIZE_IN_DIAL_OK = SIZE_DATA[0];
					var W = SIZE_DATA[1];
					var H = SIZE_DATA[2];
				} catch ( error ){};
				// если в диалоге ввода значений нажато ОК
				if( SIZE_IN_DIAL_OK )
				{
					// строка для дропдауна с размерами
					DIM_TEXT = 
					"W: "+ W.toFixed(N_TO_FIXED).toString() + " " + H_UNITS_TEXT+ " " +
					"H: "+ H.toFixed(N_TO_FIXED).toString()+ " " +V_UNITS_TEXT;
					// значения для размеров
					// для основного диалога
					if( main )
					{
						W_SIZE = W;
						H_SIZE = H;
					}
					// для пошагового диалога
					else
					{
						W_SIZE_STEP = W;
						H_SIZE_STEP = H;
					};// if-else
					// новые начальные значения для размеров
					// если ввод в основном диалоге
					if( main )
					{
						W_SIZE_INI = W.toString();
						H_SIZE_INI = H.toString();
					};// if
				}
				// если в диалоге был нажат Cancel 
				// или был некорректный ввод размера
				else
				{
					// присваиваем выбору опций фиттинга значение по умолчанию
					FIT.selection = FIT_OPTION_INI;
					return;
				};// if-else
			};// if N == FIT_SIZE_INDEX
			// добавляем в список опций пункт для значений масштабирования или размера
			// и делаем его выделенным
			REFILL_DROP( FIT, PS_FIT_OPTION_LIST, N_SPEC, DIM_TEXT );
			return;
		};// if	
		// если опция не связа с вводом значений
		if( N < FIT_SCALING_INDEX )
		{
			// перезаполняем дропдаун без конкретных значений
			// для масштабирования или размера
			REFILL_DROP( FIT, PS_FIT_OPTION_LIST, N, "" );
		};// if
		//
		return;
	};// end FIT_onChange
	// возвращаем:
	// CROP - дропдаун для обрезки 
	// TRANSP - чекбокс для прозрачности 
	// FIT - дропдаун для фиттинга 
	// CENTER - чекбокс для центрирования 
	return [CROP, TRANSP, FIT, CENTER];
};// end PLACEMENT_SETTINGS
//
// подпрограмма создания начального 
// (главного) диалога
function DIALOG_INI()
{
	var d = new Window('dialog');
	// заголовок диалога
	d.text = the_title + " v." +the_version; 
	d.orientation = 'column';
	d.alignChildren = 'center';
	// основная панель
	var mp = d.add('panel');
	mp.orientation = 'column';
	mp.alignChildren = 'left';
	// длина полей ввода для дропдаунов
	var drop_data_len = WM( 130, 130 );
	// длина имен для дропдаунов
	var name_len = WM( 18, 18 );
	// выводим имя активнонго документа 
	var DN = mp.add('statictext');
	DN.text = "Document: " + SHORT_NAME( AD_NAME, WM(25, 25) );
	DN.characters = WM( 37, 33 );
	// длина текста для чекбоксов
	var CB_TEXT_LEN = WM( 35, 35 );
	// выбор типа помещаемого файла
	FILE_TYPE = IN_DATA_DROP( mp, "Type of file to place:", TYPE_DOC_LIST, "", 0, WM(18, 15), WM(130, 156) );
	FILE_TYPE.onChange = FILE_TYPE_onChange;
	// выбор файла для помещения
	var FILE_BUTTON = BUTTON( mp, BUTTON_CHOOSE_TEXT, "File "+NOT_CHOSEN_TXT );
	FILE_BUTTON[0].onClick = FILE_BUTTON_ON_CLICK;
	// вывод полного пути файла для помещения
	var LINK_PN_TEXT = mp.add('edittext', undefined, "", {multiline: true, readonly: true} );
	LINK_PN_TEXT.text = "Graphic file path is not specified because file to place "+NOT_CHOSEN_TXT+" yet";
	LINK_PN_TEXT.minimumSize.width = LINK_PN_TEXT.maximumSize.width = WM( 265, 300 );
	LINK_PN_TEXT.maximumSize.height = LINK_PN_TEXT.minimumSize.height = WM( 50, 50 );
	// начальные значения для диапазона
	var RANGE_LIST_INI = RANGE_LIST;
	var RANGE_INI_INI = RANGE_INI;
	// начальные значения для диапазона
	// если есть выделенные текстовые фреймы
	if( TF_IN_SEL ) 
	{
		RANGE_LIST_INI = RANGE_LIST_SEL;
		RANGE_INI_INI = RANGE_INI_SEL;
	};// if
	// выбор диапазона
	var proc_range_name_len = WM( 12, 12 );
	var proc_range_dd_len = WM( 173, 183 );
	PROC_RANGE =  IN_DATA_DROP( mp, "Process range:", RANGE_LIST_INI, "", RANGE_INI_INI, proc_range_name_len, proc_range_dd_len );
	PROC_RANGE.onChange = PROC_RANGE_onChange;
	// обрабатывать ли мастер элементы (текстовые фреймы)
	var PROC_MASTER_INI = false;
	PROC_MASTER = IN_CB( mp, "Process (detach) master items (text frames)", CB_TEXT_LEN, PROC_MASTER_INI );
	// если есть текстовые фреймы в выделении или активный разворот мастер
	// начальное значение для обработки мастер элементов false
	if( TF_IN_SEL || AS_IS_MASTER ) PROC_MASTER.enabled = false;
	// обрабатывать ли заблокированные и/или невидимые текстовые фреймы
	// обрабатывать только заблокированные текстовые фреймы
	var PROC_LOCK_ONLY_TXT = "Process locked text frames";
	// обрабатывать заблокированные и невидимые текстовые фреймы
	var PROC_LOCK_INVISIBLE_TXT = "Process locked and invisible text frames";
	// если в выделении есть текстовые фреймы тогда начальное
	// предложение только заблокированные
	if( TF_IN_SEL ) 
	{
		PROC_LOCK_TXT = PROC_LOCK_ONLY_TXT;
	}
	// если в выделении нет текстовых фреймов
	else
	{
		// если версия CS5 и выше тогда могут быть невидимые текстовые фреймы
		if( VER_CS5 ) 
		{
			PROC_LOCK_TXT = PROC_LOCK_INVISIBLE_TXT;
		}
		// если ниже CS5 тогда только заблокированные
		else 
		{
			PROC_LOCK_TXT = PROC_LOCK_ONLY_TXT;
		};// if-else
	};// if-else
	PROC_LOCK = IN_CB( mp, PROC_LOCK_TXT, CB_TEXT_LEN, false );
	// если нет заблокированных текстовых фреймов в выделении
	// и при этом есть текстовые фреймы в выделении
	if( !LOCKED_IN_SEL &&  TF_IN_SEL ) 
	{
		PROC_LOCK.value = PROC_LOCK.enabled = false;
	};// if
	// обрабатывать ли текстовые фреймы на заблокированных слоях
	if( LAYERS_LOCKED )
	{
		PROC_ON_LOCKED_LAYERS = IN_CB( mp, "Process on locked and invisible layers", CB_TEXT_LEN, false );
		// если в выделении есть текстовые фреймы начальное значение false 
		if( TF_IN_SEL ) PROC_ON_LOCKED_LAYERS.enabled = false;
	};// if
	// выбор режима выполнения
	PROC_MODE = IN_DATA_DROP( mp, "Process mode:", MODE_LIST, "", 0, name_len, drop_data_len );
	// выбор типа маркирующего стиля
	MARK_STYLE = IN_DATA_DROP( mp, "Type of marking style:", MARK_STYLE_LIST, ANY_STYLE_TEXT, 0, name_len, drop_data_len );
	MARK_STYLE.onChange = MARK_STYLE_onChange;
	// выбор имени маркирующего стиля
	STYLE_NAME = IN_DATA_DROP( mp, "Marking style name:", OBJ_STYLE_LIST, "", OBJ_STYLE_INI, name_len, drop_data_len );
	// вызов подпрограммы ввода опций помещения
	var P_SET = PLACEMENT_SETTINGS
					( 
					   mp , // начальный диалог
					   true, // это главный диалог
					   CROP_OPTIONS_LIST_PDF_AI, // опции обрезки для PDF и AI
					   CROP_OPTIONS_PDF_INI, // начальное значение для обрезки PDF (Media)
					   TRANSP_INI, // начальное значение для прозрачности
					   FIT_OPTION_LIST, // список для фиттинга
					   FIT_OPTION_INI, // начальное значение для фиттинга
					   CENTER_INI // начальное значение для центрирования
					);
	CROP = P_SET[0];// дропдаун для опций обрезки
	TRANSP = P_SET[1]; // чекбокс для прозрачности
	FIT = P_SET[2]; // дропдаун для фиттинга
	CENTER = P_SET[3]; // чекбокс для центрирования
	// если некоторые идиницы измерения не разрешены
	if( BAD_UNITS )
	{ 
		// создаем панель выбора разрешенных единиц  измерения
		var UN_LINE = mp.add('group');
		UN_LINE.orientation = 'row';
		UN_LINE.alignChildren = 'left';
		var UN_DROP_LEN = WM( 51, 68 );
		H_UN = IN_DATA_DROP( UN_LINE, ("Units: " + WM("horizontal:", "horiz.:")), UN_TEXT_LIST, "", H_UNITS_DROP_INI, WM(13, 9), UN_DROP_LEN );
		H_UN.onChange = H_UN_onChange;
		V_UN = IN_DATA_DROP( UN_LINE, "vertical:", UN_TEXT_LIST, "", V_UNITS_DROP_INI, WM(6, 6), UN_DROP_LEN );
		V_UN.onChange = V_UN_onChange;
	};//if
	// ввод смещений для четных номеров
	var SHIFT_EVEN = DIM_IN ( mp, "Even:", WM(5, 5), "dx:", DX_EVEN_INI, H_UNITS_TEXT, "dy:", DY_EVEN_INI, V_UNITS_TEXT );
	// ввод смещений для НЕчетных номеров
	var SHIFT_ODD = DIM_IN ( mp, "Odd:", WM(5, 5), "dx:", DX_ODD_INI, H_UNITS_TEXT, "dy:", DY_ODD_INI, V_UNITS_TEXT );
	// 
	// подпрограмма реакции на смену горизонтальных единиц
	function H_UN_onChange()
	{
		var N = H_UN.selection.index;
		H_UNITS_TEXT = SHIFT_ODD[2].text =SHIFT_EVEN[2].text = UN_TEXT_LIST[N];
		// если в настройках фиттинга задан размер вызываем диалог для подтверждения размера
		if( (FIT.selection.index == 9) && (FIT_SCALE_SIZE == 7) ) FIT.selection = 7;
		return;
	};// end H_UN_onChange
	// 
	// подпрограмма реакции на смену вертикальных единиц
	function V_UN_onChange()
	{
		var N = V_UN.selection.index;
		V_UNITS_TEXT = SHIFT_ODD[3].text =SHIFT_EVEN[3].text = UN_TEXT_LIST[N];
		// если в настройках фиттинга задан размер вызываем диалог для подтверждения размера
		if( (FIT.selection.index == 9) && (FIT_SCALE_SIZE == 7) ) FIT.selection = 7;
		return;
	};// end H_UN_onChange
	//
	// подпрограмма реакции на смену диапазона операции
	function PROC_RANGE_onChange()
	{
		var N = PROC_RANGE.selection.index;
		// если выбрано выделение
		if( N == RANGE_INI_SEL )
		{
			// блокируем выбор мастер элементов
			PROC_MASTER.enabled = PROC_MASTER.value = false;
			// обработка только заблокированных текстовых фреймов
			PROC_LOCK.text = PROC_LOCK_ONLY_TXT;
			// если в выделении нет заблокированных объектов
			if( !LOCKED_IN_SEL &&  TF_IN_SEL ) 
			{
				// выбор заблокированных текстовых фреймов невозможен
				PROC_LOCK.value = PROC_LOCK.enabled = false;
			};// if
			// если есть заблокированные или невидимые слои
			if( LAYERS_LOCKED )
			{
				PROC_ON_LOCKED_LAYERS.value = PROC_ON_LOCKED_LAYERS.enabled = false;
			};// if
		}
		// если не выделение
		else
		{
			// разрешаем выбор обработки мастер-элементов
			// если диапазон активный разворот
			if( N == RANGE_INI )
			{
				// если активный разворот мастер
				if( AS_IS_MASTER )
				{
					// блокируем обработку мастер элементов если это было разрешено
					if( PROC_MASTER.enabled )
					{
						PROC_MASTER.enabled = PROC_MASTER.value = false;
					};// if PROC_MASTER.enabled
				}
				// если активный разворот НЕ мастер
				else
				{
					// если обработка мастер элементов не была разрешена
					if( !PROC_MASTER.enabled )
					{
						// разрешаем обработку мастер элементов
						PROC_MASTER.enabled = true;
					};// if
				};// if -else
			}
			// если диапазон НЕ активный разворот и НЕ выделение
			// (вероятно весь документ :)))
			else
			{
				// разрешаем обработку мастер элементов
				if( !PROC_MASTER.enabled ) PROC_MASTER.enabled = true;
			};// if-else
			// текст для заблокированных и невидимых объектов
			if( VER_CS5 )
			{
				if( PROC_LOCK.text != PROC_LOCK_INVISIBLE_TXT )
				{
					PROC_LOCK.text = PROC_LOCK_INVISIBLE_TXT;
				};// if
			};// if
			// разрешаем выбор заблокированных текстовых фреймов 
			if( !PROC_LOCK.enabled ) PROC_LOCK.enabled = true;
			// если есть заблокированные или невидимые слои разрешаем их выбор
			if( LAYERS_LOCKED )
			{
				if( !PROC_ON_LOCKED_LAYERS.enabled ) PROC_ON_LOCKED_LAYERS.enabled = true;
			};// if
		};// if-else			
		return;
	};// end PROC_RANGE_onChange
	//
	// подпрограмма реакции на смену типа маркирующего стиля
	function MARK_STYLE_onChange()
	{
		// индекс выбранного пункта
		var N = MARK_STYLE.selection.index;
		// если выбран любой тип стиля
		if( N == ANY_STYLE_INDEX )
		{
			// выключаем возможность выбора имени стиля
			DROP_ACTIVE_DEACTIVE (STYLE_NAME, false);
		}
		// если тип стиля кроме любого 
		else
		{
			// включаем возможность выбора имени стиля
			DROP_ACTIVE_DEACTIVE (STYLE_NAME, true);
			// выбираем список имен стилей 
			// выбран объектный стиль
			if( N == 0 ) 
			{
				var DROP_LIST = OBJ_STYLE_LIST;
				var N_INI = OBJ_STYLE_INI;
			};// if
			// выбран стиль параграфа
			if( N == 1 ) 
			{
				var DROP_LIST = PAR_STYLE_LIST;
				var N_INI = PAR_STYLE_INI;
			};// if
			// выбран стиль символа
			if( N == 2 ) 
			{
				var DROP_LIST = CH_STYLE_LIST;
				var N_INI = CH_STYLE_INI;
			};// if
			// перезаписываем список в дропдауне имен стилей
			REFILL_DROP( STYLE_NAME, DROP_LIST, N_INI, "" );
		};// if-else
		return;
	};// end MARK_STYLE_onChange
	//
	// подпрограмма получения аттрибутов документа для диалога
	// в соответствии с выбранным типом помещаемого файла
	function GET_ATTR_DIAL( N )
	{
		DOC_TYPE_DIAL = TYPE_DOC_LIST[ N ];
		FILE_TYPE_DIAL = TYPE_FILE_LIST[ N ];
		EXT_DIAL = EXT_LIST[ N ];
		return;
	};// end GET_ATTR_DIAL
	// 
	// подпрограмма реакции на смену типа файла
	function FILE_TYPE_onChange()
	{
		// индекс выбранного пункта
		var N = FILE_TYPE.selection.index;
		// получаем значения для диалога
		GET_ATTR_DIAL( N );
		// если в диалоге выбраны PDF или AI
		if( N == 0 || N == 1 )
		{
			// если выбран PDF
			// перезаполняем дропдаун с опциями обрезки для PDF и начальным значением Media
			if( N == 0 ) REFILL_DROP( CROP, CROP_OPTIONS_LIST_PDF_AI, CROP_OPTIONS_PDF_INI, "" );
			// если выбран AI
			// перезаполняем дропдаун с опциями обрезки для AI и начальным значением Bleed
			if( N == 1 ) REFILL_DROP( CROP, CROP_OPTIONS_LIST_PDF_AI, CROP_OPTIONS_AI_INI, "" );
			// если чекбокс для прозрачного фона был неактивен
			// делаем его активным
			if( !TRANSP.enabled ) TRANSP.enabled = true;
		};// if
		// если выбран INDD
		if( N == 2 ) 
		{
			// перезаполняем дропдаун с опциями обрезки для INDD и начальным значением Bleed
			REFILL_DROP( CROP, CROP_OPTIONS_LIST_INDD, CROP_OPTIONS_INDD_INI, "" );
			// чекбокс для прозрачного фона делаем неактивным
			// (для INDD он и так всегда прозрачный)
			TRANSP.enabled = false;
			// включаем его
			TRANSP.value = true;
		};// if
		// проверка расширения если файл для помещения уже выбран
		if( !FILE_NOT_CHOSEN )
		{
			// проверка расширения выбраннного файла и типа файла в диалоге
			if( FILE_EXT.toLowerCase() != EXT_DIAL )
			{
				BAD_EXT_CONFIRM( true, FILE_EXT, DOC_TYPE_DIAL );
			};// if
			// если выбран тип файла INDD проверка не совпадает ли выбранный файл
			// и файл рабочего документа
		};// if
		return;
	};// end FILE_TYPE_onChange
	//
	// создание панели кнопок (кнопка ОК, кнопка Cancel)
	var okPanel = d.add('group');
	okPanel.orientation = 'row';
	// кнопка ОК
	var okBtn = okPanel.add('button', undefined, 'OK');
	// кнопка ОК неактивна, пока не выбран файл для помещения
	okBtn.enabled = false;
	// кнопка Cancel
	var cancelBtn = okPanel.add('button', undefined, 'Cancel');
	//
	// подпрограмма реакции на выбор файла для помещения
	function FILE_BUTTON_ON_CLICK()
	{
		// получаем аттрибуты для выбранного типа файла
		GET_ATTR_DIAL( FILE_TYPE.selection.index );
		// заголовок диалога
		var dial_title = "Choose "+DOC_TYPE_DIAL+" file to place";
		// маски для типов файлов в диалоге выбора
		var files_mask = [FILE_TYPE_DIAL + " files: *" + EXT_DIAL, "All files: *.*"];
		//
		// вызов диалога выбора файла
		do 
		{
			var EXT_OK = true;
			// получаем файл из диалога
			var FILE_FROM_DIAL = File.openDialog( dial_title, files_mask );
			// если в диалоге выбора файла нажат Cancel
			if(  FILE_FROM_DIAL == null ) 
			{
				// если файл пока не выбран
				if( FILE_NOT_CHOSEN )
				{
					// сообщение в диалоге что файл не выбран
					FILE_BUTTON[1].text = "File " + NOT_CHOSEN_TXT;
					// кнопка ОК в диалоге НЕ активна
					okBtn.enabled = false;
				};// if
				return;
			}
			// если в диалоге выбора нажат ОК
			else
			{
				// файл из диалога становится рабочим файлом
				FILE = FILE_FROM_DIAL;
				// полный путь для файла из диалога
				FILE_PATH = unescape(FILE.fullName);
				var FILE_FS_NAME = unescape( FILE.fsName );
				// имя файла
				FILE_NAME = GET_NAME_FROM_PATH( FILE_FS_NAME );
				// расширение файла
				FILE_EXT = GET_EXT( FILE_NAME )[1];
				// проверка файл из диалога и документ одно и то же 
				// (актуально для выбранного INDD для помещения)
				CHECK_SAME_DOC( true );
				// проверка расширения выбранного файла и типа файла в диалоге
				if( FILE_EXT.toLowerCase() != EXT_DIAL )
				{
					EXT_OK = false;
					if( ! BAD_EXT_CONFIRM( false, FILE_EXT, DOC_TYPE_DIAL ) ) continue;
					else break; // if-else
				};// if
				break;
			};// if-else
		} while ( true );
		// выводим имя файла для помещения справа от кнопки для вызова диалога помещения
		FILE_BUTTON[1].text = SHORT_NAME( FILE_NAME, WM(15, 14) );
		// выводим полный путь файла в панели для пути
		LINK_PN_TEXT.text = FILE_FS_NAME;
		// кнопка ОК в диалоге активна
		okBtn.enabled = true;
		// файл выбран
		FILE_NOT_CHOSEN = false;
		// меняем надпись на кнопке с Выбора файла на Изменение файла		
		FILE_BUTTON[ 0 ].text = BUTTON_CHANGE_TEXT;
		return;		
	};// end FILE_BUTTON_ON_CLICK
	//
	// собственно показываем окно диалога
	var DIALOG_BUTTON = d.show();
	// если выбрана первая кнопка (ОК) выполняем
	if (DIALOG_BUTTON == 1) 
	{
		// проверка соответствия засширения выбранного файла
		// и выбранного типа документа
		GET_ATTR_DIAL( FILE_TYPE.selection.index );
		if( FILE_EXT.toLowerCase() != EXT_DIAL )
		{
			if( ! BAD_EXT_CONFIRM( false, FILE_EXT, DOC_TYPE_DIAL ) ) 
			{
				the_quit();
			};// if
		};// if
		// проверка файл из диалога и документ одно и то же
		CHECK_SAME_DOC( false );
		// текстовое значение для типа помещаемого файла
		FILE_TYPE_TEXT = FILE_TYPE.selection.text;
		// тип помещаемого файла
		FILE_TYPE = FILE_TYPE.selection.index;
		// имя файла для помещения для пошагового диалога
		FILE_NAME_DIAL = SHORT_NAME( FILE_NAME, WM(27, 27) );
		// имя документа для помещения для пошагового диалога
		AD_NAME_DIAL = SHORT_NAME( AD_NAME, WM(28, 28) );
		// текстовое значение для диапазона операции
		PROC_RANGE_TEXT = PROC_RANGE.selection.text;
		// диапазон операции
		PROC_RANGE = PROC_RANGE.selection.index;
		// обрабатывать мастер элементы
		PROC_MASTER = PROC_MASTER.value;
		// текстовое значение обрабатывать заблокированные текстовые фреймы
		PROC_LOCK_TXT = PROC_LOCK.text;
		// обрабатывать заблокированные текстовые фреймы
		PROC_LOCK = PROC_LOCK.value;
		// обрабатывать ли текстовые фреймы на заблокированных слоях
		if( LAYERS_LOCKED )
		{
			PROC_ON_LOCKED_LAYERS = PROC_ON_LOCKED_LAYERS.value;
		};// if
		// режим выполнения
		PROC_MODE = PROC_MODE.selection.index;
		// тип маркирующего стиля
		MARK_STYLE = MARK_STYLE.selection.index;
		if( MARK_STYLE == ANY_STYLE_INDEX )
		{
			ANY_STYLE_TYPE = true;
		};// if
		// текстовое значение для имени маркирующего стиля
		STYLE_NAME_TEXT = STYLE_NAME.selection.text;
		// имя маркирующего стиля
		STYLE_NAME = STYLE_NAME.selection.text;
		// текстовое значение для выбранного значения для  обрезки
		CROP_TEXT = CROP.selection.text;
		// выбранное значение для  обрезки
		CROP_INDEX = CROP.selection.index;
		// выбранное значение для прозрачного фона
		TRANSP = TRANSP.value;
		// текстовое значение для выбранного значения для подгонки во фрейме
		FIT_TEXT = FIT.selection.text;
		// выбранное значение для подгонки во фрейме
		FIT = FIT.selection.index;
		// выбранное значение для центрирования во фрейме
		CENTER = CENTER.value;
		// сдвиги для контента
		// для нечетных
		DX_ODD  = TEXT_TO_DIGIT( SHIFT_ODD[0].text );
		DY_ODD  = TEXT_TO_DIGIT( SHIFT_ODD[1].text );
		// для четных
		DX_EVEN = TEXT_TO_DIGIT( SHIFT_EVEN[0].text );
		DY_EVEN = TEXT_TO_DIGIT( SHIFT_EVEN[1].text );
		// возможный ввод единиц измерения
		if( BAD_UNITS )
		{ 
			// значения дропдаунов выбора единиц
			H_UN = H_UN.selection.index;
			V_UN = V_UN.selection.index;
			// устанавливаем выбранные единицы для документа
			H_UNITS = AD.viewPreferences.horizontalMeasurementUnits = UN_LIST[ H_UN ];
			V_UNITS =AD.viewPreferences.verticalMeasurementUnits = UN_LIST[ V_UN ];
			// текстовые значения для единиц измерения
			H_UNITS_TEXT = UN_TEXT_LIST[ H_UN ];
			V_UNITS_TEXT = UN_TEXT_LIST[ V_UN ];
			// единицы менялись (чтобы потом восствновить старые)
			UN_CHANGED = true;
		};// if
		// получаем сводку настроек в виде строки 
		ALL_SET_STR = SETTINGS_STR();
		// формирование строки для подтверждения
		var SET_CONF_STR = 
		"Settings confirmation\n\n" +
		ALL_SET_STR + "\n" +
		"Start operation?";
		// выводим запрос подтверждения
		if( !confirm( SET_CONF_STR ) )
		{
			the_quit();
		};// if
		//
		// собственно выполнение операции
		OPERATION();
	};// if
	// если в диалоге нажато Cancel
	if (DIALOG_BUTTON == 2) 
	{
		the_quit();
	};// if
	return;
};// end DIALOG_INI
//
// подпрограмма перевода логического 
// значения в текстовый вид
function YN( x )
{
	if( x ) return "Yes"
	else return "No";
};// end YN
//
// подпрограмма формирования строки с условиями поиска 
// текстовых фреймов
function CONDITIONS_STR()
{
	var S = "";
	S = S + "Process range: " + PROC_RANGE_TEXT + "\n";
	if( PROC_RANGE < 2 )
	{
		S = S + "Process (detach) master items (text frames): " + YN( PROC_MASTER ) + "\n";
	};// if
	S = S + PROC_LOCK_TXT+": "+ YN( PROC_LOCK ) + "\n";
	if( PROC_RANGE < 2 )
	{
		if( LAYERS_LOCKED )
		{
			S = S + "Process on locked and invisible layers: " + YN(PROC_ON_LOCKED_LAYERS) + "\n";
		};// if
	};// if
	S = S + "Process mode: " + MODE_LIST[PROC_MODE] +"\n";
	if( !ANY_STYLE_TYPE )
	{
		S = S + "Type of marking style: " + MARK_STYLE_LIST[ MARK_STYLE ] + "\n" +
					"Marking style name: " + STYLE_NAME_TEXT;
	}
	else
	{
		S = S + "Type of marking style: " + ANY_STYLE_TEXT;
	};// if-else
	return S
};// end CONDITIONS_STR
//
// подпрограмма вывода параметров из главного диалога для подтверждения
function SETTINGS_STR()
{
	var S = "";
	//
	S = S + "Document name: " + AD_NAME + "\n\n" +
				"Type of file to place: " + FILE_TYPE_TEXT + "\n" +
				"File name: " + FILE_NAME + "\n" +
				CONDITIONS_STR() + "\n" +
				"Crop option: " + CROP_TEXT + "\n" +
				"Transparent background: "+ YN( TRANSP ) + "\n";
	if( FIT > FIT_OPTION_MAX )
	{
		var F_STR = "Fitting: "+FIT_OPTION_LIST[ FIT_SCALE_SIZE ]+" "+FIT_TEXT+ "\n";
	}
	else
	{
		 var F_STR = "Fitting: "+FIT_TEXT + "\n";
	};// if-else
	S = S + F_STR +
				"Center content in frame: "+ YN( CENTER ) + "\n" +
				"Even numbers  dx: "+DX_EVEN.toString()+"  "+H_UNITS_TEXT+"   dy: " + DY_EVEN.toString()+" "+V_UNITS_TEXT + "\n" +
				"Odd numbers   dx: "+DX_ODD.toString()+"  "+H_UNITS_TEXT+"   dy: " + DY_ODD.toString()+" "+V_UNITS_TEXT + "\n";
	return S;
};// end SETTINGS_STR
//
// подпрограмма вывода запроса подтверждения 
// при несоответствии расширений
function BAD_EXT_CONFIRM( alert_only, file_ext, file_type )
{
	var BAD_EXT_TXT = 
	WARN_HEAD +
	"The extension ("+file_ext + ") of chosen file\n" +
	"is inappropriate for "+ file_type +"!";
	if( alert_only )
	{
		alert( BAD_EXT_TXT )
		return;
	}
	else
	{
		BAD_EXT_TXT = BAD_EXT_TXT + "\n\nContinue anyway?";
		return confirm( BAD_EXT_TXT );
	};// if-else
};// end BAD_EXT_CONFIRM
//
// подпрограмма проверки совпадения выбраннного файла
// и самого рабочего документа 
function CHECK_SAME_DOC( alert_only )
{
	// если документ не был сохранен сразу возврат
	// т.к. у несохраненного документа вообщен нет пути :)))
	if( !AD_SAVED ) return; 
	// если НЕ совпадают пути документа и выбранного файла возврат
	if( AD_PATH != FILE_PATH ) return;
	// если совпали вывод предупреждения
	var SAME_DOC_TXT = 
	WARN_HEAD +
	"The chosen file to place\n" +
	"and the document file are the same!\n";
	if( alert_only )
	{
		alert( SAME_DOC_TXT )
	}
	else
	{
		SAME_DOC_TXT =
		SAME_DOC_TXT + "\n" + WARN_END;
		alert( SAME_DOC_TXT );
		the_quit();
	};// if-else
	return;
};// end CHECK_SAME_DOC
//
// подпрограмма определения четного числа
function IS_EVEN(value) 
{
	try
	{
		// если число делится без остатка на 2
		// тогда оно четное
		if (value%2 == 0)
		{
			return true;
		}
		// иначе нечетное
		else
		{
			return false;
		};// if-else
	} catch ( error ) {};
};// end IS_EVEN
//
// подпрограмма получения 
// геометрических границ фрейма
function GBOUNDS( the_obj )
{
	// результат операции
	var OK = false;
	// границы
	var b;
	try
	{
		// получаем геометрические границы
		b = the_obj.geometricBounds;
		// если удалось получить без ошибки
		OK = true;
	}
	catch ( error ) {};// try-catch
	// возвращаем результат операции и геометрические границы
	return [OK, b];
};// end GBOUNDS
//
// ******** ПОДПРОГРАММА ОТОБРАЖЕНИЯ ФРЕЙМА
function SHOW_FRAME( the_obj ) 
{
	try 
	{
		// получаем геометрические размеры фрейма
		var obj_bounds = GBOUNDS( the_obj );
		// если размеры получены без ошибки
		if( obj_bounds[0] )
		{
			// геометрические границы = второй элемент массива из GBOUNDS
			obj_bounds = obj_bounds[1];
		}
		// если размеры не получены
		else
		{
			// невозможно показать фрейм
			CAN_SHOW = false;
			return;
		};// if-else
		// высота объекта
		var obj_height = obj_bounds[2] - obj_bounds[0];
		// ширина объекта
		var obj_width = obj_bounds[3] - obj_bounds[1];
		// активной страницей активного окна становится
		// текущая обрабатываемая страница 
		app.activeWindow.activePage = CURRENT_PAGE;
		// активное окно приложения
		var AW = app.activeWindow;
		// активная страница активного окна
		var AP = AW.activePage;
		// границы активной страницы
		var page_bounds = AP.bounds;
		// высота активной страницы
		var page_height = page_bounds[2] - page_bounds[0];
		// ширина активной страницы
		var page_width = page_bounds[3] - page_bounds[1];
		// получаем соотношения границ объекта и активной страницы
		var h_ratio  = page_height / obj_height; // по высоте
		var w_ratio = page_width / obj_width; // по ширине
		// выбираем минимальное из них
		var zoomRatio = Math.min( h_ratio, w_ratio );
		// снимаем предыдущее выделение
		DESELECT();
		// если текстовый фрейм НЕ является мастер элементом
		if( !MASTER_ITEM )
		{
			// выделяем текстовый фрейм
			app.select( the_obj, SelectionOptions.replaceWith );
			// вписываем активную страницу в активное окно
			AW.zoom( ZoomOptions.FIT_PAGE );
			// масштабируем отображение активной страницы
			AW.zoomPercentage = AW.zoomPercentage * zoomRatio;
		}
		// если текстовый фрейм является мастер элементом
		else
		{
			// вписываем активную страницу в активное окно
			// и ничего не выделяем
			AW.zoom( ZoomOptions.FIT_PAGE );
		};// if-else
		// если не было ошибки считаем, что изображение 
		// можно показать
		CAN_SHOW = true;
	}
	// если была ошибка при показе окна с изображением
	// считаем, что изображение нельзя показать
	catch ( error ) 
	{
		CAN_SHOW = false;
	};// try-catch
	return;
};// end SHOW_FRAME
//
// подпрограмма выполнения операции
function OPERATION()
{
	// устанавливаем начальные параметры обрезки в соответствии 
	// с выбранными в основном диалоге опциями
	if( FILE_TYPE == 0 || FILE_TYPE == 1 )
	{
		// если PDF или AI
		var FILE_PDF_AI = true;
		// массив обрезки для PDF или AI для пошагового диалога
		var CROP_OPTION_LIST_INI = CROP_OPTIONS_LIST_PDF_AI;
	}
	// если INDD
	else
	{
		// обрезка для INDD
		var FILE_PDF_AI = false;
		// массив обрезки для для INDD для пошагового диалога
		var CROP_OPTION_LIST_INI = CROP_OPTIONS_LIST_INDD;
	};// if-else
	//
	// подпрограмма обработки текстового фрейма
	function PROC_TF( the_frame, parent_page, is_master )
	// the_frame - обрабатываемый фрейм
	// parent_page - страница, на которой находится фрейм
	// (для возможной обработки номера страницы в текстовом фрейме)
	// is_master - является ли мастер элементом
	{
		// текст во фрейме
		var TXT;
		// номер из текста во фрейме
		var NUM;
		// является ли номер четным
		var NUM_IS_EVEN = false;
		// результат перевода текста в число
		var BAD_NUM = false;
		// смещения для изображения во фрейме
		var DX_PROC = 0.;
		var DY_PROC = 0.;
		// опции обрезки для операции помещения во фрейм
		var CROP_INDEX_PROC = CROP_INDEX;
		// прозрачность фона для фрейма
		var TRANSP_PROC = TRANSP;
		// настройки фиттинга для помещения во фрейм
		var FIT_PROC = FIT_CURRENT;
		// центрирование для изображения во фрейме
		var CENTER_PROC = CENTER;
		// масштабирование для фиттинга
		var W_SCALE_PROC = W_SCALE;
		var H_SCALE_PROC = H_SCALE;
		// размеры для фиттинга
		var W_SIZE_PROC = W_SIZE;
		var H_SIZE_PROC = H_SIZE;
		// кнопка нажатая в диалоге для пошагового режима
		var BUTTON_STEP_DIAL;
		// проблемный текстовый фрейм
		var BAD_FRAME = false;
		//
		// подпрограма получения ориентации (угла поворота) фрейма
		function FRAME_ORIENT( F )
		{
			// если была проблема с получением угла поворота
			BAD_FRAME = false;
			try
			{
				var S = "Frame's ";
				try
				{
					var A = F.rotationAngle;
				}
				catch ( error )
				{
					BAD_FRAME = true;
				};// try-catch
				switch ( A )
				{
					// голова фрейма направлена вверх
					case ( 0. ):
					{
						S = S + "head: UP"; 
						break;
					};// case 0.
					// голова фрейма направлена вниз
					case ( 180. ):
					{
						S = S + "head: DOWN"; 
						break; 
					};// case 180.
					// голова фрейма направлена вправо
					case ( -90. ):
					{
						S = S + "head: RIGHT"; 
						break;
					};// case -90.
					// голова фрейма направлена влево
					case ( 90. ):
					{
						S = S + "head: LEFT"; 
						break;
					};// case 90.
					// если угол поворота не совпал с направлениями
					// выводим только угол поворота
					default:
					{
						S = S + "angle: " + ( A.toFixed(2) ).toString(); 
						break;
					};// default
				};// case A
				return S;
			}
			catch ( error )
			{
				BAD_FRAME = true;
				return "";
			};// if-else
		};// end FRAME_ORIENT
		//
		try
		{
			// информация об ориентации фрейма
			var FRAME_ORIENT_TEXT = FRAME_ORIENT( the_frame );
			// если проблема с получением информации о фрейме выход
			if( BAD_FRAME )
			{
				N_SKIP++;
				N_ERR++;
				return;
			};// if
			// имя страницы
			var PAGE_NAME = parent_page.name;
			// получаем текстовое содержимое фрейма
			TXT = the_frame.contents;
			// если получен код номера страницы
			if( TXT.charCodeAt(0) == 24 )
			{
				// используем имя страницы на которой находится текстовый фрейм
				// оно чаще всего является числом :)))
				TXT = parent_page.name;
			};// if
			// получаем число из текста
			NUM = parseInt( TXT );
			// если то что получено из текста не число
			// или оно не может быть номером страницы
			if(isNaN( NUM ) || NUM <= 0 ) BAD_NUM = true;
		}
		catch ( error )
		{
			BAD_NUM = true;
		};// try-catch
		// если номер без ошибок
		if( !BAD_NUM )
		{
			// определяем четное ли число
			NUM_IS_EVEN = IS_EVEN( NUM );
			// смещения для четного номера
			if( NUM_IS_EVEN )
			{
				DX_PROC = DX_EVEN;
				DY_PROC = DY_EVEN;
			}
			// смещения для НЕчетного номера
			else
			{
				DX_PROC = DX_ODD;
				DY_PROC = DY_ODD;
			};// if-else
		}
		else
		{
			// если ошибка с номером и автоматический режим
			// тогда пропускаем этот фрейм
			if( PROC_MODE == 0 )
			{
				N_ER++;
				N_SKIP++;
				return;
			};// if
		};// if-else
		//
		// подпрограмма получения опций обрезки
		function SET_CROP( N )
		{
			var CROP;
			// опции обрезки для PDF и AI
			if( FILE_PDF_AI )
			{
				if(N == 0 ) CROP = PDFCrop.CROP_ART;
				if( N == 1 ) CROP = PDFCrop.CROP_PDF;
				if( N == 2 ) CROP = PDFCrop.CROP_TRIM;
				if( N == 3 ) CROP = PDFCrop.CROP_BLEED;
				if( N == 4 ) CROP = PDFCrop.CROP_MEDIA;
				if( N == 5 ) CROP = PDFCrop.CROP_CONTENT_VISIBLE_LAYERS;
				if( N == 6 ) CROP = PDFCrop.CROP_CONTENT_ALL_LAYERS;
				// применяем опции обрезки для PDF и AI
				try
				{
					app.pdfPlacePreferences.pdfCrop = CROP;
				} catch ( error ) {};
			}
			// опции обрезки для INDD
			else
			{
				if( N == 0 ) CROP = ImportedPageCropOptions.CROP_CONTENT;
				if( N == 1 ) CROP = ImportedPageCropOptions.CROP_BLEED;
				if( N == 2 ) CROP = ImportedPageCropOptions.CROP_SLUG;
				// применяем опции обрезки для INDD
				try
				{
					app.importedPageAttributes.importedPageCrop = CROP;
				} catch ( error ) {};
			};// if-else
			return;
		};// end SET_CROP
		//
		// подпрограмма назначения номера помещаемой графической страницы
		function SET_NUM( N )
		{
			// если PDF или AI
			if( FILE_PDF_AI )
			{
				app.pdfPlacePreferences.pageNumber = N;
			}
			// если INDD
			else
			{
				app.importedPageAttributes.pageNumber = N;
			};// if-else
			return;
		};// end SET_NUM
		//
		// подпрограмма выполнения фиттинга
		function SET_FIT( N, G )
		{
			try
			{
				// если заданы настройки для размера или масштабирования
				if( N > FIT_OPTION_MAX )
				{
					N = FIT_SCALE_SIZE;
				};// if
				// use preference settings
				if( N == 0 ) 
				{ 
					// сразу выходим, ничего не делаем
					return;
				};// if 0
				// fit content to frame
				if( N == 1 ) 
				{
					G.fit( FitOptions.CONTENT_TO_FRAME );
				};// if 1
				// fit content to frame proportionally
				if( N == 2 ) 
				{
					G.fit( FitOptions.PROPORTIONALLY );
				};// if 2
				// fit frame to content
				if( N == 3 )
				{
					G.fit( FitOptions.FRAME_TO_CONTENT );
				};// if 3
				// fill frame proportionally
				if( N == 4 )
				{
					G.fit( FitOptions.FILL_PROPORTIONALLY );
				};// if 4
				// 100% content scaling
				if( N == 5 ) 
				{
					SCALE( G, 100., 100. );
				};// if 5
				// specified content scaling
				if( N == 6 ) 
				{
					SCALE( G, W_SCALE_PROC, H_SCALE_PROC );
				};// if 6
				// specified content size
				if( N == 7 ) 
				{
					SIZE( G, W_SIZE_PROC, H_SIZE_PROC );
				};// if 7
				//
				// подпрограмма ввода нового размера
				function SIZE( G, S_W, S_H )
				{
					// если версия CS5 и новее
					if( VER_CS5 )
					{
						// подпрограмма получения пунктов
						function PT( x, u )
						{
							// если единицы пункты сразу возврат исходного значения
							if( u == 2054188905 ) return x;
							// если единицы измерения НЕ агаты
							if (u != 2051106676 )
							{
								// если цицеро
								if( u == 2053335395 )
								{
									// корректируем
									return x * 12.78788;
								};// if
								// переводим цифры размеров в единицы измерения документа
								// если НЕ десятичные дюймы
								if( u != 2053729892 )
								{
									var a = new UnitValue(x, u );
								}
								// если десятичные дюймы 
								else
								{
									// корректируем
									var a = new UnitValue(x, MeasurementUnits.INCHES );
								};// if-else
								// переводим размеры в пункты для функции трансформации
								a = a.as("pt");
							}
							else
							// если единицы агаты
							{
								//  корректируем
								var a = (x /14. ) * 72.;
							};// if-else
							return a;
						};// end PT
						//
						// пространство координат
						var the_cs = CoordinateSpaces.INNER_COORDINATES;
						// центр трансформации: левый верхний угол фрейма
						var the_ancor = AnchorPoint.TOP_LEFT_ANCHOR;
						// заменять старые значения размеров
						var the_rm = ResizeMethods.REPLACING_CURRENT_DIMENSIONS_WITH;
						// получаем пункты из значений в диалоге
						var H_VAL = PT(S_W, H_UNITS );
						var V_VAL = PT(S_H, V_UNITS );
						// размеры как массив для ввода
						var the_size = [ H_VAL, V_VAL ];
						G.resize
						(
							the_cs, // пространство координат
							the_ancor, // центр трансформации
							the_rm, // заменять старые значения размеров
							the_size,// размеры как массив
							true, // трансформировать объекты по отдельности (а он и так один :)))
							false // ввод размеров в пунктах
						);
					}
					// если версия CS4 и старше
					else
					{
						// попдпрограмма перевода в пункты из текстового значения единиц
						function PT_UN_TEXT( x, u )
						{
							// если единицы пункты сразу возврат
							if( u == "pt" ) return x;
							// если единицы НЕ агаты
							if( u != "ag" )
							{
								// если цицеро
								if( u == "ci" )
								{
									// корректируем
									return x * 12.78788;
								};// if
								// если НЕ десятичные дюймы
								if( u != "ind" )
								{
									var a = new UnitValue(x, u );
								}
								// если десятичные дюймы 
								else
								{
									// корректируем
									var a = new UnitValue(x, "in" );
								};// if-else
								a = a.as("pt");
							}
							// если единицы агаты
							else
							{
								//  корректируем
								var a = (x /14. ) * 72.;
							};// if-else
							return a;
						};// end PT_UN_TEXT
						//
						// сначала ставим масштаб 100%
						SCALE( G, 100., 100. );
						//получаем начальный размер изображения в пунктах
						var the_size = G.resolve( AnchorPoint.BOTTOM_RIGHT_ANCHOR, CoordinateSpaces.INNER_COORDINATES, false );
						// переводим координаты в текст
						the_size = the_size + "";
						// ширина из координат (текст)
						var the_w = "";
						// высота из координат (текст)
						var the_h = "";
						// дошли до запятой в координатах
						var comma = false;
						// цикл по символам текста координат
						for( var i = 0; i < the_size.length; i++ )
						{
							// текущий символ
							var d = the_size[i];
							// если получена запятая
							if( d == "," )
							{
								comma = true;
								continue;
							};// if
							// до запятой получаем ширину
							if( !comma )
							{
								the_w = the_w + d;
							}
							// после запятой получаем высоту
							else
							{
								the_h = the_h + d;
							};// if-else
						};// for
						// переводим старые размеры из текста в число
						the_w = parseFloat( the_w );
						the_h = parseFloat( the_h );
						// переводим цифры новых размеров в единицы измерения документа
						// для ранних версий используем текстовые значения для единиц
						var H_VAL = PT_UN_TEXT( S_W, H_UNITS_TEXT );
						var V_VAL = PT_UN_TEXT( S_H, V_UNITS_TEXT );
						// рассчитываем масштабирование для получения нового заданного размера
						var d_w = ( H_VAL / the_w ) * 100.;
						var d_h = ( V_VAL / the_h ) * 100.;
						// масштабируем для получения новых размеров
						SCALE( G, d_w, d_h );
					};// if-else
					return;
				};// end SIZE
				//
				// подпрограмма масштабирования
				function SCALE( G, S_W, S_H )
				{
					// пространство координат
					var the_cs = CoordinateSpaces.PARENT_COORDINATES;
					// центр трансформации: левый верхний угол фрейма
					var the_ancor = AnchorPoint.TOP_LEFT_ANCHOR
					//  матрица трансформаций для масштабирования
					var the_scale = app.transformationMatrices.add({ horizontalScaleFactor: S_W*0.01,  verticalScaleFactor: S_H*0.01 });
					// собственно трансформация
					G.transform
					(
						the_cs, // пространство координат
						the_ancor, // центр трансформации
						the_scale, //  матрица трансформаций для масштабирования
						MatrixContent.SCALE_VALUES, // замещение возможных прошлых трансформаций
						true // учитывать единицы измерений
					);
					return;					
				};// end SCALE
			}
			catch ( error )
			{
				N_ER++;
			};// try-catch
			return;
		};// end SET_FIT
		//
		// подпрограмма получения информации о заблокированности
		// и невидимости фрейма или слоя
		function LOCK_INFO( F )
		{
			// фрейм заблокирован
			var F_LOCKED = false;
			// фрейм невидимый
			var F_INVISIBLE = false;
			// слой на котором фрейм заблокирован
			var LAYER_LOCKED = false;
			// слой на котором фрейм невидимый
			var LAYER_INVISIBLE = false;
			// строка сообщения
			var S = "";
			// результирующее значение 
			// (есть ли вообще что-либо заблокированное или невидимое)
			var LOCK = false;
			try
			{
				// проверка блокировки фрейма
				F_LOCKED = F.locked;
				// проверка видимости фрейма
				if( F.hasOwnProperty( "visible" ) )
				{
					F_INVISIBLE = !F.visible;
				};// if
				// проверка блокировки слоя
				LAYER_LOCKED = F.itemLayer.locked;
				// проверка видимости слоя
				LAYER_INVISIBLE = !F.itemLayer.visible;
				// если есть что-либо заблокированное или невидимое
				if( F_LOCKED || F_INVISIBLE  || LAYER_LOCKED || LAYER_INVISIBLE )
				{
					LOCK = true;
					// если фрейм заблокирован или невидимый
					if( F_LOCKED || F_INVISIBLE )
					{
						S = S + "Frame is ";
						if( F_LOCKED ) S = S + "locked";
						if( F_LOCKED && F_INVISIBLE ) S = S + ", ";
						if( F_INVISIBLE ) 
						{
							S = S + "invisible";
							CAN_SHOW = false;
						};// if
						S = S + "! ";
					};// if
					// если слой заблокирован или невидимый
					if( LAYER_LOCKED || LAYER_INVISIBLE )
					{
						S = S + "Layer is ";
						if( LAYER_LOCKED ) S = S + "locked";
						if( LAYER_LOCKED && LAYER_INVISIBLE ) S = S + ", ";
						if( LAYER_INVISIBLE ) 
						{
							S = S + "invisible";
							CAN_SHOW = false;
						};// if
						S = S + "! ";
					};// if
				};// if
			} catch (error) {};
			return [ LOCK, S];
		};// end LOCK_INFO
		//
		// подпрограмма создания диалога для пошагового
		// режима операции
		function STEP_DIAL()
		{
			// показываем текущий фрейм
			SHOW_FRAME (the_frame);
			// создаем окно диалога
			var d = new Window('dialog');
			// положение окна диалога если он уже выводился
			if( STEP_DIAL_LOCATION[0] )
			{
				try
				{
					// присваиваем предыдущее положение окна диалога
					d.location = STEP_DIAL_LOCATION[1];
				} catch ( error ) {};// try-catch
			};// if
			 // заголовок диалога
			d.text = the_title + " v." +the_version;
			d.orientation = 'column';
			d.alignChildren = 'center';
			// создаем параметры для цветного текста в диалоге (вдруг понадобится :)))
			// красный текст
			var RED_TEXT = d.graphics.newPen (d.graphics.PenType.SOLID_COLOR, [1, 0, 0, 1], 1);
			// синий текст
			var BLUE_TEXT = d.graphics.newPen (d.graphics.PenType.SOLID_COLOR, [0, 0, 1, 1], 1);
			//
			// подпрограмма создания линии в диалоге
			function DLINE( where, len )
			{
				var LINE = where.add('panel');
				LINE.bounds = [0, undefined, len , undefined];
				return LINE;
			};// end DLINE
			//
			// ширина верхней группы
			var d_w = WM (300, 340);
			// блок информации о всем диапазоне операции
			var h_blok = d.add('group');
			h_blok.orientation = 'column';
			h_blok.alignChildren = 'left';
			DDW( h_blok, d_w );
			DTEXT( h_blok, "Document: "+ AD_NAME_DIAL );
			DTEXT( h_blok, "File to place: "+ FILE_NAME_DIAL );
			var RANGE_TEXT = PROC_RANGE_TEXT;
			if( PROC_RANGE == 0 ) RANGE_TEXT = "Document";
			if( PROC_RANGE == 1 ) RANGE_TEXT = "Active spread";
			if( PROC_RANGE == 2 ) RANGE_TEXT = "Selection";
			DTEXT( h_blok, "Process range: "+ RANGE_TEXT +"  Frames in range: " +N_FRAMES );
			DTEXT( h_blok, "Processed: "+ N_PROC.toString() + "  Skipped: " + N_SKIP.toString() + "  Errors: " + N_ER.toString() );
			// строка данных для текущего фрейма
			var DATA_LINE = d.add('group');
			DATA_LINE.orientation = 'row';
			DATA_LINE.alignChildren = 'left';
			DDW( DATA_LINE, d_w );
			// информация о странице на которой находится фрейм
			if( PROC_RANGE == 0 )
			{
				// о странице если диапазон весь документ
				var DATA_LINE_TEXT = "Page: " + PAGE_NAME + "   ";
			}
			else
			{
				// о развороте если диапазон выделение или активный разворот
				var DATA_LINE_TEXT = "Spread: " + AS_NAME + "   ";
			};// if-else
			// информация об ориентации фрейма
			DATA_LINE_TEXT = DATA_LINE_TEXT + FRAME_ORIENT_TEXT;
			DTEXT( DATA_LINE, DATA_LINE_TEXT );
			// информация является ли фрейм мастер-объектом
			if( is_master )
			{
				DTEXT( DATA_LINE, "Master item!" ).graphics.foregroundColor = BLUE_TEXT;
			};// i
			// строка для сообщений о видимости и заблокированности
			var F_LOCK = LOCK_INFO( the_frame );
			if( F_LOCK[0] )
			{
				var lock_invisible_line = d.add('group'); 
				lock_invisible_line.orientation = 'row';
				lock_invisible_line.alignChildren = 'left';
				DDW( lock_invisible_line, d_w );
				DTEXT( lock_invisible_line, F_LOCK[1] ).graphics.foregroundColor = BLUE_TEXT;
			};// if
			// строка для сообщений о возможности показать фрейм
			if( !CAN_SHOW )
			{
				var CAN_NOT_SHOW_TEXT = "Can not show the frame!";
				var can_not_show_line = d.add('group'); 
				can_not_show_line.orientation = 'row';
				can_not_show_line.alignChildren = 'left';
				DDW( can_not_show_line, d_w );
				DTEXT( can_not_show_line, CAN_NOT_SHOW_TEXT ).graphics.foregroundColor = RED_TEXT;
			};// if
			// проводим линию
			DLINE( d, d_w )
			//
			// группа для номера во фрейме
			var f_num_line = d.add('group');
			f_num_line.orientation = 'row';
			f_num_line.alignChildren = 'left';
			DDW( f_num_line, d_w );
			// выводим номер страницы из содержимого фрейма
			var NUM_IN_FRAME = "Number in frame: ";
			// если цифра очень длинная
			if( NUM > 999 ) NUM_IN_FRAME = "In frame: ";
			// если номер в текстовом фрейме в порядке
			if( !BAD_NUM ) 
			{
				// строка с номером во фрейме
				var NUM_IN_FRAME = NUM_IN_FRAME + NUM.toString();
				// начальное значение для ввода номера графической страницы = номер во фрейме
				var NUM_INI = NUM;
			}
			else
			{
				// строка с сообщением что номер не определен
				var NUM_IN_FRAME = NUM_IN_FRAME + "N/A";
				// начальное значение для ввода номера графической страницы = 1
				var NUM_INI = 1;
			};// if-else
			// собственно выводим строку с номером во фрейме
			DTEXT( f_num_line, NUM_IN_FRAME );
			var num_field_w = WM(35, 30);
			// если цифра очень длинная увеличиваем размер поля ввода
			if( NUM > 999 ) num_field_w = WM(60, 60)
			// поле для ввода номера страницы
			var f_1 = NUM_FIELD( f_num_line, "Graphic page to place: ", NUM_INI, num_field_w, "" )[0];
			f_1.onChanging = f_1_onChanging;
			// подпрограмма реакции на ввод номера страницы
			function f_1_onChanging()
			{
				digit_on_Changing( f_1, NUM_INI, NUM_INI );
				var f_1_text = f_1.text;
				var f_1_num = TEXT_TO_DIGIT( f_1_text );
				// если ноль или отрицательное число
				if( f_1_num <= 0 )
				{
					alert( "Page number must be positive!" );
					f_1.text = NUM_INI;
				};// if
				// если введена десятичная точка или запятая
				for( var i = 0; i < f_1_text.length; i++ )
				{
					if( (f_1_text[i] == ".") || (f_1_text[i] == ",") )
					{
						alert( "Page number must be integer!" );
						f_1.text = NUM_INI;
						break;
					};// if
				};// for
				return;				
			};// end f_1_onChange
			// панель для опций помещения
			var mp = d.add('panel');
			mp.orientation = 'column';
			mp.alignChildren = 'left';
			// начальное присвоение масштабирования (из основного диалога)
			W_SCALE_STEP = W_SCALE;// по горизонтали (ширина)
			H_SCALE_STEP = H_SCALE;// по вертикали (высота)
			// начальное присвоение размеров (из основного диалога)
			W_SIZE_STEP = W_SIZE;// по горизонтали (ширина)
			H_SIZE_STEP = H_SIZE;// по вертикали (высота)
			// собственно вызов подпрограммы построения панели
			var P_SET = PLACEMENT_SETTINGS
			( 
				mp, // где (в каком диалоге) выводится блок
				false, // является ли диалог главным
				CROP_OPTION_LIST_INI,  // список опций обрезки
				CROP_INDEX, // начальная опция обрезки
				TRANSP, // начальная опция для прозрачности
				FIT_OPTION_LIST, // список опций для фиттинга
				FIT, // начальное значение для фиттинга
				CENTER // начальное значение для центрирования
			);
			// если выбранный тип файла INDD отключаем выбор прозрачности
			// в смысле жестко включаем без возможности отключения :)))
			if( ! FILE_PDF_AI )
			{
				P_SET[1].enabled = false;
				P_SET[1].value = true;
			};// if
			// если в главном диалоге были выбраны настройки для фиттинга
			// с вводом масштабирования или размера
			function REFILL_FIT_OPTION_LIST()
			{
				if( FIT > 5 )
				{
					// перезаполняем массив опций для фиттинга
					REFILL_DROP( P_SET[2], FIT_OPTION_LIST, FIT, FIT_TEXT );
				};// if
			};// end REFILL_FIT_OPTION_LIST
			// 
			// перезаполняем массив опций фиттинга в соответствии с начальными значениями
			REFILL_FIT_OPTION_LIST();
			//
			// начальные присвоения смещений (из основного диалога)
			var DX_STEP_INI = DX_PROC.toString();
			var DY_STEP_INI = DY_PROC.toString();
			// создаем поля ввода для смещений
			var SHIFT = DIM_IN ( mp, "Shift:", WM( 5, 5 ), "dx:", DX_STEP_INI, H_UNITS_TEXT, "dy:", DY_STEP_INI, V_UNITS_TEXT );
			//
			// группа кнопок внизу
			var B_GROUP = d.add('group');
			B_GROUP.orientation = 'column';
			B_GROUP.alignChildren = 'center';
			// строка для первых трех кнопок
			var B_GROUP_LINE_1 = B_GROUP.add('group');
			B_GROUP_LINE_1.orientation = 'row';
			B_GROUP_LINE_1.alignChildren = 'center';
			// кнопка переустановки данных Reset data
			// данные приравниваются к тем значениям, которые были получены в главном диалоге
			var R_DATA_B = BUTTON( B_GROUP_LINE_1, R_TEXT, "" )[0];
			R_DATA_B.onClick = RESET_DATA_ON_CLICK;
			// кнопка пропуска текущего фрейма Skip frame
			var SKIP_B = BUTTON( B_GROUP_LINE_1, S_TEXT, "" )[0];
			SKIP_B.onClick = SKIP_ON_CLICK;
			// кнопка завершения всей операции Qiut operation
			var QUIT_B = BUTTON( B_GROUP_LINE_1, Q_TEXT, "" )[0];
			d.cancelElement = QUIT_B;
			// кнопка ОК
			var OK_B = BUTTON( B_GROUP, OK_TEXT, "" )[0];
			d.defaultElement = OK_B;
			// 
			// подпрограмма реакции на нажатие кнопки Reset data
			function RESET_DATA_ON_CLICK()
			{
				// поле ввода номера графической страницы
				f_1.text = NUM_INI;
				// дропдаун опций обрезки
				P_SET[0].selection = CROP_INDEX;
				// чекбокс для прозрачности фона
				if( P_SET[1].enabled )
				{
					P_SET[1].value = TRANSP;
				};// if
				// дропдаун фиттинга
				REFILL_FIT_OPTION_LIST();
				P_SET[2].selection = FIT;
				// чекбокс для центрирования контента
				P_SET[3].value = CENTER;
				// поля ввода для сдвигов
				SHIFT[0].text = DX_STEP_INI;
				SHIFT[1].text = DY_STEP_INI;
				return;
			};// end RESET_DATA_ON_CLICK
			//
			// подпрограмма реакции на нажатие кнопки Skip frame
			function SKIP_ON_CLICK()
			{
				// окно закрывается
				d.close();
				// выход из подпрограммы
				return;
			};// end SKIP_ON_CLICK
			//
			// показываем окно диалога
			BUTTON_STEP_DIAL = d.show();
			// запоминаем положение окна диалога
			STEP_DIAL_LOCATION[0] = true;
			STEP_DIAL_LOCATION[1] = d.location;
			// BUTTON_STEP_DIAL = 0 нажата кнопка Skip frame
			// BUTTON_STEP_DIAL = 1 нажата кнопка ОК
			// BUTTON_STEP_DIAL = 2 нажата кнопка Qiut operation
			//
			// если нажата кнопка ОК
			if( BUTTON_STEP_DIAL == 1 )
			{
				// номер графической страницы для помещения
				var RET_PAGE_NUM = parseInt( TEXT_TO_DIGIT( f_1.text ) );
				// опции обрезки
				var RET_CROP = P_SET[0].selection.index;
				// прозрачность фона
				var RET_TRANSP = P_SET[1].value;
				// фиттинг
				var RET_FIT = P_SET[2].selection.index;
				// центровка
				var RET_CENTER = P_SET[3].value;
				// смещения для контента
				var RET_DX = TEXT_TO_DIGIT( SHIFT[0].text );
				var RET_DY = TEXT_TO_DIGIT( SHIFT[1].text );
				//
				return [ RET_PAGE_NUM, RET_CROP, RET_TRANSP, RET_FIT, RET_CENTER, RET_DX, RET_DY ];
			};// if
			// если нажата кнопка Quit operation
			if( BUTTON_STEP_DIAL == 2 )
			{
				// снимаем выделение
				DESELECT();
				// завершение операции
				the_exit("");
			};// if
		};// end STEP_DIAL
		//
		// подпрограмма получения графического содержимого фрейма
		function GET_CONTENT( F )
		{
			// если PDF или AI
			if( FILE_PDF_AI )
			{
				 return F.pdfs[0];
			}
			// если INDD
			else
			{
				return F.importedPages[0];
			};// if-else
		};// end GET_CONTENT
		//
		// подпрограмма получения номера помещенной графической страницы
		function GRAPH_PAGE_NUM( G )
		{
			// если PDF или AI
			if( FILE_PDF_AI )
			{
				return G.pdfAttributes.pageNumber;
			}
			// если INDD
			else
			{
				return G.pageNumber;
			};// if-else
		};// end GRAPH_PAGE_NUM
		//
		// подпрограмма перемещения изображения
		function MOVE( obj, dx, dy )
		{
			// если заданные перемещения = 0 сразу выход
			if( (dx == 0.) && (dy == 0.) ) return;
			try
			{
				// собственно перемещение объекта
				obj.move(undefined, [dx, dy]);
			}
			catch ( error ) 
			{
				N_ER++;
			};// try-catch
			return;
		};// end MOVE
		//
		// если режим выполнения пошаговый
		if( PROC_MODE == 1 )
		{
			// вызов подпрограммы диалога
			var STEP_DIAL_DATA = STEP_DIAL();
			// если в диалоге нажата кнопка ОК 
			// для операции используем значения из диалога
			if( BUTTON_STEP_DIAL == 1 )
			{
				// номер страницы из диалога
				NUM = STEP_DIAL_DATA[0];
				BAD_NUM = false;
				// опции обрезки из диалога
				CROP_INDEX_PROC = STEP_DIAL_DATA[1];
				// прозрачность фона из диалога
				TRANSP_PROC = STEP_DIAL_DATA[2];
				// настройки фиттинга из диалога
				FIT_PROC = STEP_DIAL_DATA[3];
				// настройка центрирования из диалога
				CENTER_PROC = STEP_DIAL_DATA[4];
				// настройки смещения из диалога
				DX_PROC = STEP_DIAL_DATA[5];
				DY_PROC = STEP_DIAL_DATA[6];
				// масштабирование для фиттинга из диалога
				W_SCALE_PROC = W_SCALE_STEP;
				H_SCALE_PROC = H_SCALE_STEP;
				// размеры для фиттинга из диалога
				W_SIZE_PROC = W_SIZE_STEP;
				H_SIZE_PROC = H_SIZE_STEP;
			};// if BUTTON_STEP_DIAL == 1
			// если в диалоге нажата кнопка пропустить фрейм
			if( BUTTON_STEP_DIAL == 0 )
			{
				N_SKIP++;
				return;
			};// if
		};// if PROC_MODE == 1
		//
		// если ошибка при получении номера страницы
		if( BAD_NUM )
		{
			N_SKIP++;
			// аварийный выход из подпрограммы 
			// только для автоматического режима
			return;
		};// if
		try
		{
			// применяем значение номера страницы помещаемого файла
			SET_NUM( NUM );
			// устанавливаем опции обрезки
			SET_CROP( CROP_INDEX_PROC );
			// устанавливаем опцию прозрачного фона (для PDF и AI)
			if( FILE_PDF_AI ) 
			{
				app.pdfPlacePreferences.transparentBackground = TRANSP_PROC;
			}
			// если текстовый фрейм является мастер элементом 
			if( is_master )
			{
				// тогда сначала отделяем от мастера
				the_frame = the_frame.override( parent_page );
			};// if
			// удаляем текст из текстового фрейма
			the_frame.characters.everyItem().remove();
			// запоминаем ID текстового фрейма
			var the_frame_id = the_frame.id;
			// собственно помещаем графическую страницу из файла
			var G_page = the_frame.place( FILE );
			// определяем текущий фрейм как объект документа
			// потому что он больше не является ТЕКСТОВЫМ фреймом
			the_frame = AD.pageItems.itemByID( the_frame_id );
			// получаем графическое содержимое 
			G_page = GET_CONTENT( the_frame );
			// проверяем реальный номер импортированной страницы
			if( GRAPH_PAGE_NUM( G_page ) != NUM )
			{
				// если помещенный номер графической страницы
				// не совпадет с заданным считаем это ошибкой
				N_ER++;
			};// if
			// выполнение опций фиттинга
			SET_FIT( FIT_PROC, G_page );
			// выполнение центрирования
			if( CENTER_PROC )
			{
				G_page.fit( FitOptions.CENTER_CONTENT );
			};// if
			// выполнение перемещения
			MOVE( G_page, DX_PROC, DY_PROC );
			N_PROC++;
		}
		catch( error )
		{
			N_ER++;
		};// try-catch
		return;
	};// end PROC_TF
	//
	// подпрограмма  обработки массива текстовых фреймов
	//  arr - входящий массив текстовых фреймов
	// parent_page - страница, на которой находятся фреймы
	// is_master - являются ли объекты мастер элементом
	function PROC_TF_ARR( arr, parent_page, is_master )
	{
		// цикл по текстовым фреймам
		for( var i=0; i < arr.length; i++ )
		{
			PROC_TF( arr[i], parent_page, is_master );
		};// for
		return;
	};// end PROC_TF_ARR
	//
	// подпрограмма получения текущей страницы
	function GET_CURRENT_PAGE( N )
	{
		// если диапазон операции активный разворот 
		// и активный разворот является мастер-разворотом
		if( PROC_RANGE == RANGE_INI && AS_IS_MASTER )
		{
			// текущая страница = страница мастер-разворота
			var CP = AS.pages[ N ];
		}
		// если обычный разворот или диапазон весь документ
		// кроме мастер-разворотов
		else
		{
			// текущая страница = страница документа
			var CP = AD.pages[ N ];
		};// if-else
		return CP;
	};// end GET_CURRENT_PAGE
	//
	// подпрограмма получения текстовых фреймов в определенном диапазоне
	// (на развороте или полосе)
	function GET_TEXT_FRAMES_IN_RANGE( arr_in, arr_out )
	// the_range - объект для поиска текстовых фреймов (разворот или страница)
	// arr_in - массив объектов, среди которых проводтся поиск
	// arr_out - выходной массив текстовых фреймов
	{
		var CHECK_CONDITIONS = true;
		GET_TEXT_FRAMES ( arr_in, arr_out, CHECK_CONDITIONS );
		return;
	};// end GET_TEXT_FRAMES_IN_RANGE
	//
	// подпрограмма получения текстовых фреймов на странице
	function GET_TF_ON_PAGE( N )
	{
		CURRENT_PAGE = GET_CURRENT_PAGE( N );
		// массив текстовых фреймов на текущей странице
		FRAMES[N] = new Array();
		// массив текстовых фреймов (мастер элементов) на текущей странице
		MASTER_FRAMES[N] = new Array();
		// элементы не являются мастер элементами
		MASTER_ITEM = false;
		// получаем текстовые фреймы на текущей странице
		GET_TEXT_FRAMES_IN_RANGE( CURRENT_PAGE.allPageItems, FRAMES[N] );
		// если задана обработка мастер элементов
		if( PROC_MASTER )
		{
			// элементы являются мастер элементами
			MASTER_ITEM = true;
			// получаем текстовые фреймы среди мастер элементов	
			GET_TEXT_FRAMES_IN_RANGE( CURRENT_PAGE.masterPageItems, MASTER_FRAMES[N] );
		};// if
		return;		
	};// end GET_TF_ON_PAGE
	//
	// подпрограмма обработки текстовых фреймов на странице
	function PROC_TF_ON_PAGE( N )
	{
		CURRENT_PAGE = GET_CURRENT_PAGE( N );
		// обработка элементов которые не являются мастер элементами
		MASTER_ITEM = false;
		PROC_TF_ARR( FRAMES[ N ], CURRENT_PAGE, MASTER_ITEM );
		// если задана обработка мастер элементов
		if( PROC_MASTER )
		{
			MASTER_ITEM = true;
			PROC_TF_ARR( MASTER_FRAMES[ N ], CURRENT_PAGE, MASTER_ITEM );
		};// if
		return;
	};// end PROC_TF_ON_PAGE
	//
	// подпрограмма поиска и обработки текстовых фреймов
	// в заданном диапазоне страниц документа
	function PROC_TF_IN_RANGE( START, END )
	{
		// цикл по страницам диапазоне
		for( var i = START; i < END; i++ )
		{
			// получаем текстовые фреймы 
			// на текущей странице диапазона
			GET_TF_ON_PAGE( i );
		};// for
		if( PROC_MASTER ) CHECK_MASTER_ITEMS();
		// проверяем рабочий массив (не пустой ли)
		CHECK_FRAMES_NUMBER();
		// обработка фреймов в заданном диапазоне
		for( var i = START; i < END; i++ )
		{
			PROC_TF_ON_PAGE( i );
		};// for
		return;		
	};// end PROC_TF_IN_RANGE
	//
	// подпрограмма проверки совпадающих мастер элементов на разворотах 
	// (т.е. элементов, которые могут сразу оказаться на нескольких полосах разворота)
	// например, если одна часть мастер элемента находится на одной полосе, а другая часть
	// на другой полосе мастер-разворота, то при поиске мастер элементов на обычном развороте
	// эти части одного мастер элемента будут распознаны как 2 объекта, поэтому должен остаться
	// из двух один :)))
	function CHECK_MASTER_ITEMS()
	{
		// подпрограмма для проверки разворота
		function CHECK_SPREAD( the_spr )
		{
			try
			{
				// количество страниц на развороте
				var N = the_spr.pages.length;
				// если всего 1 страница возврат
				if( N == 1 ) return;
				// начальная страница разворота
				var start = SPR_START_PAGE( the_spr );
				// конечная страница разворота
				var end = start + N;
				// цикл по массивам для страниц
				for( var i = start; i < end-1; i++ )
				{
					// текущий массив
					var current_master_arr = MASTER_FRAMES[ i ];
					// цикл по объектам текущего массива
					for( var k = 0; k < current_master_arr.length; k++ )
					{
						// текущий объект
						var current_obj = current_master_arr[ k ];
						// ID текущего объекта
						var current_obj_id = current_obj.id;
						// цикл по следующим массивам страниц на развороте
						for( var j = i + 1; j < end; j++ )
						{
							// следующий массив
							var next_master_arr = MASTER_FRAMES[ j ];
							var arr_out = new Array();
							// цикл по объектам следующего массива
							for( var m = 0; m < next_master_arr.length; m++ )
							{
								// объект следующего массива
								var next_obj = next_master_arr[ m ];
								// ID объекта следующего массива
								var next_obj_id = next_obj.id;
								// проверка совпадения ID
								if( !(next_obj_id == current_obj_id) )
								{
									// если нет совпадения помещаем объект
									// следующего массива в выходной массив
									arr_out.push( next_obj );
								};// if
							};// for m
							// переворачиваем полученный промежуточный массив
							arr_out.reverse();
							// переприсваиваем следующий массив
							MASTER_FRAMES[ j ] = arr_out;
						};// for j
					};// for k
				};// for i
			} catch ( error ) {};
			return;
		};// end CHECK_SPREAD
		//
		// если диапазон активный разворот
		if( PROC_RANGE == RANGE_INI )
		{
			CHECK_SPREAD( AS );
		};// if
		// если диапазон весь документ
		if( PROC_RANGE == 0 )
		{
			// цикл по разворотам документа
			for( var i = 0; i < AD.spreads.length; i++ )
			{
				CHECK_SPREAD( AD.spreads[i] );
			};// for
		};// if
		return;
	};// end CHECK_MASTER_ITEMS
	//
	// если диапазон операции выделение
	if( PROC_RANGE == RANGE_INI_SEL )
	{
		// элементы не являются мастер элементами
		// поскольку находятся в выделении
		MASTER_ITEM = false;
		// значение для текущей страницы (формальное)
		CURRENT_PAGE = AS.pages[ 0 ];
		// получаем из выделенных текстовых фреймов 
		// только те, которые соответствуют выбранному маркирующему стилю
		// рабочий массив FRAMES
		var CHECK_CONDITIONS = true;
		GET_TEXT_FRAMES ( SEL_FRAMES, FRAMES, CHECK_CONDITIONS );
		// проверяем рабочий массив (не пустой ли)
		CHECK_FRAMES_NUMBER();
		// обработка фреймов в выделении
		PROC_TF_ARR( FRAMES, CURRENT_PAGE, MASTER_ITEM );
	};// if PROC_RANGE == RANGE_INI_SEL
	//
	// если диапазон операции активный разворот
	if( PROC_RANGE == RANGE_INI )
	{
		// получаем индекс начальной страницы активного разворота
		START_PAGE = SPR_START_PAGE( AS );
		// количество страниц на активном развороте
		N_PAGES = AS.pages.length;
		// получаем индекс конечной страницы на активном развороте
		END_PAGE = START_PAGE + N_PAGES;
		// обработка текстовых фреймов на активном развороте
		if( AS_IS_MASTER )
		{
			START_PAGE = 0;
			END_PAGE = N_PAGES;
		};// if
		PROC_TF_IN_RANGE( START_PAGE, END_PAGE );
	};// if PROC_RANGE == RANGE_INI 
	//
	// если диапазон операции документ
	if( PROC_RANGE == 0 )
	{
		// индекс начальной страницы документа
		START_PAGE = 0;
		// количество страниц в документе
		N_PAGES = AD.pages.length
		// получаем индекс конечной страницы документа
		END_PAGE = N_PAGES;
		// обработка текстовых фреймов в документе
		PROC_TF_IN_RANGE( START_PAGE, END_PAGE );
	};// if PROC_RANGE == 0
	// снимаем выделение если был пошаговый режим
	if( PROC_MODE == 1 ) DESELECT();
	// звершение операции
	the_exit("");
};// end OPERATION
//
// подпрограмма снятия выделения в документе
function DESELECT()
{
	try
	{
		// в выделении активного документа ничего нет :)))
		AD.selection = null;
	} catch ( error ) {};
	return;
};// end DESELECT
//
// подпрограмма получения номера начальной страницы на развороте
function SPR_START_PAGE( the_spr )
{
	// ID самой первой (0) страницы на развороте
	var the_id = the_spr.pages[0].id;
	// цикл по всем страницам документа
	for( var i=0; i < AD.pages.length; i++ )
	{
		// если совпали ID возвращаем значение индекса
		// он равен индексу страницы в документе
		if( AD.pages[i].id == the_id ) return i;
	};// for
	return 0;
};// end SPR_START_PAGE
//
// подпрограмма проверки для мастер элемента
// является ли он отдельным объектом (не в группе и т.п.)
function CHECK_MASTER_ITEM( the_obj )
{
	var parent_name = the_obj.parent.constructor.name;
	// если родительский объект мастер разворот или обычная страница все ОК возвращаемся
	if( (parent_name == "MasterSpread") || (parent_name == "Page") ) return;
	// если родительский объект НЕ мастер разворот и не страница
	// формирование сообщения
	var S = 
	WARN_START+
	"whiсh are master items belong to\n"+
	"complex objects (groups etc.)!\n"+
	WARN_END;
	// вывод сообщения
	alert( S );
	// завершение операции
	the_quit();
};// end CHECK_MASTER_ITEM
//
// подпрограмма завершения работы 
// при наличии цепочек более 1 фрейма
function CHECK_STORY( the_obj )
{
	var story_ok = true;
	try
	{
		// цепочка в которой находится фрейм
		var the_parent_story = the_obj.parentStory;
		// количество фреймов в цепочке, в которой находится фрейм
		var f_num = 0;
		// учитываем разные версии индизайна
		// для новых версий
		if( the_parent_story.hasOwnProperty ("textContainers") )
		{
			f_num = the_parent_story.textContainers.length;
		}
		// для старых версий
		else
		{
			f_num = the_parent_story.textFrames.length;
		};// if-else
		// если количество более 1
		if( f_num > 1 ) story_ok = false;
	} catch (error) {};// try-catch
	// если фрейм в цепочке с более 1 фреймом
	if( !story_ok )
	{
		// формирование сообщения
		var S =
		WARN_START+
		"belong to stories containing more than 1 text frame!\n"+
		WARN_END;
		// вывод сообщения
		alert( S );
		// завершение операции
		the_quit();
	};// if
	return;
};// end CHECK_STORY
//
// подпрограмма проверки размера массива
// текстовых фреймов для операции
function CHECK_FRAMES_NUMBER()
{
	var S = WARN_HEAD;
	// проверка для диапазона выделение
	if( PROC_RANGE == RANGE_INI_SEL )
	{
		// число текстовых фреймов в рабочем массиве
		N_FRAMES = FRAMES.length;
	}
	else
	// проверка для диапазона активный разворот или весь документ
	{
		// цикл по страницам 
		for( var i = START_PAGE; i < END_PAGE; i++ )
		{
			// суммируем число текстовых фреймов на каждой странице
			N_FRAMES = N_FRAMES + FRAMES[i].length;
			// если задана обработка мастер элементов
			if( PROC_MASTER )
			{
				// добавляем число текстовых фреймов как мастер элементов на странице
				N_FRAMES = N_FRAMES + MASTER_FRAMES[i].length;
			};// if
		};// for
	};// if - else
	// если не найдено фреймов для обработки
	if( N_FRAMES == 0 )
	{
		// формирование сообщения
		S = S +
		"No text frames conforming\n" +
		"the chosen conditions were found!\n\n" +
		CONDITIONS_STR();
		if( BAD_POS )
		{
			S = S + "\n\n"+BAD_POS_TEXT;
		};// if
		// вывод сообщения
		alert( S );
		// завершение операции
		the_quit();
	}
	// если фреймы для обработки есть
	else
	{
		// если были исключены фреймы из-за неправильного положения
		if( BAD_POS )
		{
			S = S +
			BAD_POS_TEXT + "\n\n" +
			"Continue anyway?"
			if( !confirm( S ) )
			{
				// если нет подтверждения завершение операции
				the_quit();
			};// if
		};// if
	};// if-else
	return
};// end CHECK_FRAMES_NUMBER
//
// подпрограмма получения имени из пути
function GET_NAME_FROM_PATH( P )
{
	var P_LEN = P.length;
	var NAME = "";
	// цикл по символам начиная с последнего
	for( var i = P_LEN -1; i >=0; i-- )
	{
		var C = P[i];
		// если найден слэш считаем что имя закончилось
		if( C == "\\" || C == "/") break;
		NAME = C + NAME;
	};// for i
	return NAME;
};// end GET_NAME_FROM_PATH
//
// подпрограмма получения расширения файла и имени без расширения
function GET_EXT( N )
{
	// расширение на выходе
	var EXT = "";
	// имя на выходе
	var N_ONLY = "";
	// окончание поиска расширения
	var STOP_EXT = false;
	// цикл по символам начиная с последнего
	for( var i = N.length-1; i >= 0; i-- )
	{
		var N_i = N[i];
		if( !STOP_EXT )
		{
			EXT = N_i + EXT;
			// если найдена точка считаем что поиск расширения закончен
			if( N_i == "." ) STOP_EXT = true;
		};// if
		// если найден слэш считаем что имя закончилось
		// начался путь, если он был на входе :)))
		if( ( N_i == "/") || ( N_i == "\\") ) break;
		N_ONLY = N_i + N_ONLY;
	};// for
	// если расширения вообще нет
	if( ( N_ONLY == EXT ) && ( EXT. match(".") != "." ) )
	{
		// пустая строка для расширения
		// для возможного вывода в диалоге
		EXT = "empty string";
	};// if
	// возвращаем
	// N_ONLY - только имя без расширения
	// EXT - расширение
	return [ N_ONLY, EXT ];	
};// end GET_EXT
//
// подпрограмма вывода сообщения о результатах работы
function REPORT()
{
	// формирование сообщения
	var S = "Report\n\n" +
	ALL_SET_STR + "\n" +
	"Frames in range: " + N_FRAMES + "\n" +
	"Processed: " + N_PROC + "\n" +
	"Skipped: " + N_SKIP + "\n" +
	"Errors: "+ N_ER;
	// вывод сообщения
	alert( S );
	return;
};// end REPORT
//
// подпрограмма присвоения значения переменной
// в зависимости от файловой системы
function WM( W, M )
{
	// если Windows
	if( WFS ) return W
	// если Macintosh
	else return M;
};// end WM
